// -*- c++ -*-
// sigslot.h: Signal/Slot classes
// 
// Written by Sarah Thompson (sarah@telergy.com) 2002.
//
// License: Public domain. You are free to use this code however you like,
// with the proviso that the author takes on no responsibility or liability
// for any use.
//
// USING THE LIBRARY
//   See the full documentation at http://sigslot.sourceforge.net/
//

/*
 * sigslot libraryΤԡǤꡢʲ
 * ꥸʥsigslot libraryȰۤʤäƤ롣
 *
 * 1. ꥸʥsigslot libraryǤϡåǥ󥰥ݥꥷλˤ
 *    ۤƱȤ߹ޤ뤬ܥСǤϡå
 *    ֥ȤŪͿAPIѹƤ롣
 *    ѹϡlibochushaΥѥ˻ꤵ륹åǥ󥰥ݥꥷ
 *    libochushaѤץΥѥ˻ꤵ륹åǥ
 *    ݥꥷۤʤ뤳Ȥ⤽ʤᡢְ㤨;Ϥ̵
 *    ȤŪȤƤ롣
 * 2. ꥸʥsigslot libraryǤϡpthreadǤѤƤå
 *    ̾mutexǤ롣pthreadmutexrecursive lockǤ뤳
 *    ݾڤƤʤᡢsigslot libraryξʤȤ
 *    multi_threaded_globalݥꥷưʤб뤿
 *    ᡢrecursive lock¸륪֥ȤåѤ褦ѹ
 *    
 * 3. 12ѹη̡ꥸʥsigslot libraryȤϴߴˤʤ
 *    Ƥ뤿ᡢnamespaceѤ
 *
 * $Id: sigslot.h,v 1.10 2006/12/25 11:45:34 fuyu Exp $
 */

#ifndef _LIBOCHUSHA_SIGSLOT_H_
#define _LIBOCHUSHA_SIGSLOT_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/lock.h"

#include <algorithm>
#include <set>
#include <list>


using namespace std;
namespace Ochusha
{
  class _connection_base;
  class has_slots;

  class _signal_base
  {
  protected:
    Lock *lock;
    list<_connection_base *> connected_slots;   


  public:
    _signal_base(Lock *lock = NULL);
    virtual ~_signal_base();

    void disconnect_all();
    void disconnect(has_slots *slot_holder);
    void slot_disconnect(has_slots *slot);
    void slot_duplicate(const has_slots *oldtarget, has_slots *newtarget);
  };


  class has_slots
  {
  private:
    set<_signal_base *> senders;


  protected:
    Lock *lock;


  public:
    has_slots(Lock *lock = NULL);
    has_slots(const has_slots &hs);
    virtual ~has_slots();

    void signal_connect(_signal_base *sender);
    void signal_disconnect(_signal_base *sender);
    void disconnect_all();
  };


  class _connection_base
  {
  protected:
    has_slots *object;


  public:
    _connection_base(const _connection_base &orig) : object(orig.object) {}
    _connection_base(has_slots *object) : object(object) {}
    virtual ~_connection_base() {}
    inline has_slots *get_dest() const { return object; }

    virtual _connection_base *clone() = 0;
    virtual _connection_base *duplicate(has_slots *newdest) = 0;
  };


  /**********************************************************************
   * ʤsignal/slot						*
   **********************************************************************/

  class _connection_base0 : public _connection_base
  {
  public:
    _connection_base0(const _connection_base0 &orig) : _connection_base(orig) {}
    _connection_base0(has_slots *object) : _connection_base(object) {}
    ~_connection_base0() {}
    virtual void emit() = 0;
  };


  template <class TD>
  class _connection0 : public _connection_base0
  {
  private:
    void (TD::*memfun)();


  public:
    _connection0(const _connection0 &orig)
      : _connection_base0(orig), memfun(orig.memfun) {}
    _connection0(TD *object, void (TD::*memfun)())
      : _connection_base0(object), memfun(memfun) {}
    ~_connection0() {}

    virtual _connection_base *clone()
    { return new _connection0<TD>(*this); }

    virtual _connection_base *duplicate(has_slots *newdest)
    { return new _connection0<TD>(dynamic_cast<TD *>(newdest), memfun); }

    inline void emit() { ((dynamic_cast<TD *>(object))->*memfun)(); }
  };


  class Signal0 : public _signal_base
  {
  public:
    Signal0(Lock *lock = NULL) : _signal_base(lock) {}
    ~Signal0() {}

    template <class TD>
    void connect(TD *target, void (TD::*memfun)())
    {
      LockBlock lock_obj(lock);
      _connection0<TD> *conn = new _connection0<TD>(target, memfun);
      connected_slots.push_back(conn);
      target->signal_connect(this);
    }

    void emit();
    inline void operator()() { emit(); }
  };


  /**********************************************************************
   * Ĥsignal/slot						*
   **********************************************************************/

  template <class TA1>
  class _connection_base1 : public _connection_base
  {
  public:
    _connection_base1(const _connection_base1 &orig) : _connection_base(orig) {}
    _connection_base1(has_slots *object) : _connection_base(object) {}
    ~_connection_base1() {}
    virtual void emit(TA1 a1) = 0;
  };


  template <class TD, class TA1>
  class _connection1 : public _connection_base1<TA1>
  {
  private:
    void (TD::*memfun)(TA1);


  public:
    _connection1(const _connection1 &orig)
      : _connection_base1<TA1>(orig), memfun(orig.memfun) {}
    _connection1(TD *object, void (TD::*memfun)(TA1))
      : _connection_base1<TA1>(object), memfun(memfun) {}
    ~_connection1() {}

    virtual _connection_base *clone()
    { return new _connection1<TD, TA1>(*this); }

    virtual _connection_base *duplicate(has_slots *newdest)
    { return new _connection1<TD, TA1>(dynamic_cast<TD *>(newdest), memfun); }

    virtual void emit(TA1 a1) { (dynamic_cast<TD *>(this->object)->*memfun)(a1); }
  };


  template <class TA1>
  class signal1 : public _signal_base
  {
  public:
    signal1(Lock *lock = NULL) : _signal_base(lock) {}
    ~signal1() {}

    template <class TD>
    void connect(TD *target, void (TD::*memfun)(TA1))
    {
      LockBlock lock_obj(lock);
      _connection1<TD, TA1> *conn = new _connection1<TD, TA1>(target, memfun);
      connected_slots.push_back(conn);
      target->signal_connect(this);
    }

    inline void operator()(TA1 a1) { emit(a1); }

    void emit(TA1 a1)
    {
      LockBlock lock_obj(lock);

      list<_connection_base *>::const_iterator it = connected_slots.begin();
      list<_connection_base *>::const_iterator itEnd = connected_slots.end();
      while (it != itEnd)
	{
	  list<_connection_base *>::const_iterator itNext = it;
	  ++itNext;
	  dynamic_cast<_connection_base1<TA1> *>(*it)->emit(a1);
	  it = itNext;
	}
    }
  };


  /**********************************************************************
   * Ĥsignal/slot						*
   **********************************************************************/


  template <class TA1, class TA2>
  class _connection_base2 : public _connection_base
  {
  public:
    _connection_base2(const _connection_base2 &orig) : _connection_base(orig) {}
    _connection_base2(has_slots *object) : _connection_base(object) {}
    ~_connection_base2() {}
    virtual void emit(TA1 a1, TA2 a2) = 0;
  };


  template <class TD, class TA1, class TA2>
  class _connection2 : public _connection_base2<TA1, TA2>
  {
  private:
    void (TD::*memfun)(TA1, TA2);


  public:
    _connection2(const _connection2 &orig)
      : _connection_base2<TA1, TA2>(orig), memfun(orig.memfun) {}
    _connection2(TD *object, void (TD::*memfun)(TA1, TA2))
      : _connection_base2<TA1, TA2>(object), memfun(memfun) {}
    ~_connection2() {}

    virtual _connection_base *clone()
    { return new _connection2<TD, TA1, TA2>(*this); }
    virtual _connection_base *duplicate(has_slots *newdest)
    { return new _connection2<TD, TA1, TA2>(dynamic_cast<TD *>(newdest), memfun); }

    inline void emit(TA1 a1, TA2 a2)
    { (dynamic_cast<TD *>(this->object)->*memfun)(a1, a2); }
  };


  template <class TA1, class TA2>
  class signal2 : public _signal_base
  {
  public:
    signal2(Lock *lock = NULL) : _signal_base(lock) {}
    ~signal2() {}

    template <class TD>
    void connect(TD *target, void (TD::*memfun)(TA1, TA2))
    {
      LockBlock lock_obj(lock);
      _connection2<TD, TA1, TA2> *conn
	= new _connection2<TD, TA1, TA2>(target, memfun);
      connected_slots.push_back(conn);
      target->signal_connect(this);
    }

    inline void operator()(TA1 a1, TA2 a2) { emit(a1, a2); }

    void emit(TA1 a1, TA2 a2)
    {
      LockBlock lock_obj(lock);

      list<_connection_base *>::const_iterator it = connected_slots.begin();
      list<_connection_base *>::const_iterator itEnd = connected_slots.end();
      while (it != itEnd)
	{
	  list<_connection_base *>::const_iterator itNext = it;
	  ++itNext;
	  dynamic_cast<_connection_base2<TA1, TA2> *>(*it)->emit(a1, a2);
	  it = itNext;
	}
    }
  };


  /**********************************************************************
   * Ĥsignal/slot						*
   **********************************************************************/


  template <class TA1, class TA2, class TA3>
  class _connection_base3 : public _connection_base
  {
  public:
    _connection_base3(const _connection_base3 &orig) : _connection_base(orig) {}
    _connection_base3(has_slots *object) : _connection_base(object) {}
    ~_connection_base3() {}
    virtual void emit(TA1 a1, TA2 a2, TA3 a3) = 0;
  };


  template <class TD, class TA1, class TA2, class TA3>
  class _connection3 : public _connection_base3<TA1, TA2, TA3>
  {
  private:
    void (TD::*memfun)(TA1, TA2, TA3);


  public:
    _connection3(const _connection3 &orig)
      : _connection_base3<TA1, TA2, TA3>(orig), memfun(orig.memfun) {}
    _connection3(TD *object, void (TD::*memfun)(TA1, TA2, TA3))
      : _connection_base3<TA1, TA2, TA3>(object), memfun(memfun) {}
    ~_connection3() {}

    virtual _connection_base *clone()
    { return new _connection3<TD, TA1, TA2, TA3>(*this); }

    virtual _connection_base *duplicate(has_slots *newdest)
    { return new _connection3<TD, TA1, TA2, TA3>(dynamic_cast<TD *>(newdest), memfun); }

    void emit(TA1 a1, TA2 a2, TA3 a3)
    { (dynamic_cast<TD *>(this->object)->*memfun)(a1, a2, a3); }
  };


  template <class TA1, class TA2, class TA3>
  class signal3 : public _signal_base
  {
  public:
    signal3(Lock *lock = NULL) : _signal_base(lock) {}
    ~signal3() {}

    template <class TD>
    void connect(TD *target, void (TD::*memfun)(TA1, TA2, TA3))
    {
      LockBlock lock_obj(lock);
      _connection3<TD, TA1, TA2, TA3> *conn
	= new _connection3<TD, TA1, TA2, TA3>(target, memfun);
      connected_slots.push_back(conn);
      target->signal_connect(this);
    }

    inline void operator()(TA1 a1, TA2 a2, TA3 a3)
    { emit(a1, a2, a3); }

    void emit(TA1 a1, TA2 a2, TA3 a3)
    {
      LockBlock lock_obj(lock);

      list<_connection_base *>::const_iterator it = connected_slots.begin();
      list<_connection_base *>::const_iterator itEnd = connected_slots.end();
      while (it != itEnd)
	{
	  list<_connection_base *>::const_iterator itNext = it;
	  ++itNext;
	  dynamic_cast<_connection_base3<TA1, TA2, TA3> *>(*it)
	    ->emit(a1, a2, a3);
	  it = itNext;
	}
    }
  };


  /**********************************************************************
   * ͤĤsignal/slot						*
   **********************************************************************/


  template <class TA1, class TA2, class TA3, class TA4>
  class _connection_base4 : public _connection_base
  {
  public:
    _connection_base4(const _connection_base4 &orig) : _connection_base(orig) {}
    _connection_base4(has_slots *object) : _connection_base(object) {}
    ~_connection_base4() {}
    virtual void emit(TA1 a1, TA2 a2, TA3 a3, TA4 a4) = 0;
  };


  template <class TD, class TA1, class TA2, class TA3, class TA4>
  class _connection4 : public _connection_base4<TA1, TA2, TA3, TA4>
  {
  private:
    void (TD::*memfun)(TA1, TA2, TA3, TA4);


  public:
    _connection4(const _connection4 &orig)
      : _connection_base4<TA1, TA2, TA3, TA4>(orig), memfun(orig.memfun) {}
    _connection4(TD *object, void (TD:: *memfun)(TA1, TA2, TA3, TA4))
      : _connection_base4<TA1, TA2, TA3, TA4>(object), memfun(memfun) {}
    ~_connection4() {}

    virtual _connection_base *clone()
    { return new _connection4<TD, TA1, TA2, TA3, TA4>(*this); }

    virtual _connection_base *duplicate(has_slots *newdest)
    { return new _connection4<TD, TA1, TA2, TA3, TA4>(dynamic_cast<TD *>(newdest), memfun); }

    void emit(TA1 a1, TA2 a2, TA3 a3, TA4 a4)
    { (dynamic_cast<TD *>(this->object)->*memfun)(a1, a2, a3, a4); }
  };


  template <class TA1, class TA2, class TA3, class TA4>
  class signal4 : public _signal_base
  {
  public:
    signal4(Lock *lock = NULL) : _signal_base(lock) {}
    ~signal4() {}

    template <class TD>
    void connect(TD *target, void (TD::*memfun)(TA1, TA2, TA3, TA4))
    {
      LockBlock lock_obj(lock);
      _connection4<TD, TA1, TA2, TA3, TA4> *conn
	= new _connection4<TD, TA1, TA2, TA3, TA4>(target, memfun);
      connected_slots.push_back(conn);
      target->signal_connect(this);
    }

    inline void operator()(TA1 a1, TA2 a2, TA3 a3, TA4 a4)
    { emit(a1, a2, a3, a4); }

    void emit(TA1 a1, TA2 a2, TA3 a3, TA4 a4)
    {
      LockBlock lock_obj(lock);

      list<_connection_base *>::const_iterator it = connected_slots.begin();
      list<_connection_base *>::const_iterator itEnd = connected_slots.end();
      while (it != itEnd)
	{
	  list<_connection_base *>::const_iterator itNext = it;
	  ++itNext;
	  dynamic_cast<_connection_base4<TA1, TA2, TA3, TA4> *>(*it)
	    ->emit(a1, a2, a3, a4);
	  it = itNext;
	}
    }
  };


  /**********************************************************************
   * ޤĤsignal/slot						*
   **********************************************************************/

  template <class TA1, class TA2, class TA3, class TA4, class TA5>
  class _connection_base5 : public _connection_base
  {
  public:
    _connection_base5(const _connection_base5 &orig) : _connection_base(orig) {}
    _connection_base5(has_slots *object) : _connection_base(object) {}
    ~_connection_base5() {}
    virtual void emit(TA1 a1, TA2 a2, TA3 a3, TA4 a4, TA5 a5) = 0;
  };


  template <class TD, class TA1, class TA2, class TA3, class TA4, class TA5>
  class _connection5 : public _connection_base5<TA1, TA2, TA3, TA4, TA5>
  {
  private:
    void (TD::*memfun)(TA1, TA2, TA3, TA4, TA5);


  public:
    _connection5(const _connection5 &orig)
      : _connection_base5<TA1, TA2, TA3, TA4, TA5>(orig), memfun(orig.memfun) {}
    _connection5(TD *object, void (TD:: *memfun)(TA1, TA2, TA3, TA4, TA5))
      : _connection_base5<TA1, TA2, TA3, TA4, TA5>(object), memfun(memfun) {}
    ~_connection5() {}

    virtual _connection_base *clone()
    { return new _connection5<TD, TA1, TA2, TA3, TA4, TA5>(*this); }

    virtual _connection_base *duplicate(has_slots *newdest)
    { return new _connection5<TD, TA1, TA2, TA3, TA4, TA5>(dynamic_cast<TD *>(newdest), memfun); }

    void emit(TA1 a1, TA2 a2, TA3 a3, TA4 a4, TA5 a5)
    { (dynamic_cast<TD *>(this->object)->*memfun)(a1, a2, a3, a4, a5); }
  };


  template <class TA1, class TA2, class TA3, class TA4, class TA5>
  class signal5 : public _signal_base
  {
  public:
    signal5(Lock *lock = NULL) : _signal_base(lock) {}
    ~signal5() {}

    template <class TD>
    void connect(TD *target, void (TD::*memfun)(TA1, TA2, TA3, TA4, TA5))
    {
      LockBlock lock_obj(lock);
      _connection5<TD, TA1, TA2, TA3, TA4, TA5> *conn
	= new _connection5<TD, TA1, TA2, TA3, TA4, TA5>(target, memfun);
      connected_slots.push_back(conn);
      target->signal_connect(this);
    }

    inline void operator()(TA1 a1, TA2 a2, TA3 a3, TA4 a4, TA5 a5)
    { emit(a1, a2, a3, a4, a5); }

    void emit(TA1 a1, TA2 a2, TA3 a3, TA4 a4, TA5 a5)
    {
      LockBlock lock_obj(lock);

      list<_connection_base *>::const_iterator it = connected_slots.begin();
      list<_connection_base *>::const_iterator itEnd = connected_slots.end();
      while (it != itEnd)
	{
	  list<_connection_base *>::const_iterator itNext = it;
	  ++itNext;
	  dynamic_cast<_connection_base5<TA1, TA2, TA3, TA4, TA5> *>(*it)
	    ->emit(a1, a2, a3, a4, a5);
	  it = itNext;
	}
    }
  };


  /**********************************************************************
   * ϻĤsignal/slot						*
   **********************************************************************/

  template <class TA1, class TA2, class TA3, class TA4, class TA5, class TA6>
  class _connection_base6 : public _connection_base
  {
  public:
    _connection_base6(const _connection_base6 &orig) : _connection_base(orig) {}
    _connection_base6(has_slots *object) : _connection_base(object) {}
    ~_connection_base6() {}
    virtual void emit(TA1 a1, TA2 a2, TA3 a3, TA4 a4, TA5 a5, TA6 a6) = 0;
  };


  template <class TD,
	    class TA1, class TA2, class TA3, class TA4, class TA5, class TA6>
  class _connection6 : public _connection_base6<TA1, TA2, TA3, TA4, TA5, TA6>
  {
  private:
    void (TD::*memfun)(TA1, TA2, TA3, TA4, TA5, TA6);


  public:
    _connection6(const _connection6 &orig)
      : _connection_base6<TA1, TA2, TA3, TA4, TA5, TA6>(orig),
	memfun(orig.memfun) {}
    _connection6(TD *object, void (TD:: *memfun)(TA1, TA2, TA3, TA4, TA5, TA6))
      : _connection_base6<TA1, TA2, TA3, TA4, TA5, TA6>(object),
	memfun(memfun) {}
    ~_connection6() {}

    virtual _connection_base *clone()
    { return new _connection6<TD, TA1, TA2, TA3, TA4, TA5, TA6>(*this); }

    virtual _connection_base *duplicate(has_slots *newdest)
    { return new _connection6<TD, TA1, TA2, TA3, TA4, TA5, TA6>(dynamic_cast<TD *>(newdest), memfun); }

    void emit(TA1 a1, TA2 a2, TA3 a3, TA4 a4, TA5 a5, TA6 a6)
    { (dynamic_cast<TD *>(this->object)->*memfun)(a1, a2, a3, a4, a5, a6); }
  };


  template <class TA1, class TA2, class TA3, class TA4, class TA5, class TA6>
  class signal6 : public _signal_base
  {
  public:
    signal6(Lock *lock = NULL) : _signal_base(lock) {}
    ~signal6() {}

    template <class TD>
    void connect(TD *target, void (TD::*memfun)(TA1, TA2, TA3, TA4, TA5, TA6))
    {
      LockBlock lock_obj(lock);
      _connection6<TD, TA1, TA2, TA3, TA4, TA5, TA6> *conn
	= new _connection6<TD, TA1, TA2, TA3, TA4, TA5, TA6>(target, memfun);
      connected_slots.push_back(conn);
      target->signal_connect(this);
    }

    inline void operator()(TA1 a1, TA2 a2, TA3 a3, TA4 a4, TA5 a5, TA6 a6)
    { emit(a1, a2, a3, a4, a5, a6); }

    void emit(TA1 a1, TA2 a2, TA3 a3, TA4 a4, TA5 a5, TA6 a6)
    {
      LockBlock lock_obj(lock);

      list<_connection_base *>::const_iterator it = connected_slots.begin();
      list<_connection_base *>::const_iterator itEnd = connected_slots.end();
      while (it != itEnd)
	{
	  list<_connection_base *>::const_iterator itNext = it;
	  ++itNext;
	  dynamic_cast<_connection_base6<TA1, TA2, TA3, TA4, TA5, TA6> *>(*it)
	    ->emit(a1, a2, a3, a4, a5, a6);
	  it = itNext;
	}
    }
  };


  /**********************************************************************
   * Ĥbool signal/slot					*
   **********************************************************************/


  template <class TA1, class TA2>
  class _bconnection_base2 : public _connection_base
  {
  public:
    _bconnection_base2(const _bconnection_base2 &orig)
      : _connection_base(orig) {}
    _bconnection_base2(has_slots *object) : _connection_base(object) {}
    ~_bconnection_base2() {}
    virtual bool emit(TA1 a1, TA2 a2) = 0;
  };


  template <class TD, class TA1, class TA2>
  class _bconnection2 : public _bconnection_base2<TA1, TA2>
  {
  private:
    bool (TD::*memfun)(TA1, TA2);


  public:
    _bconnection2(const _bconnection2 &orig)
      : _bconnection_base2<TA1, TA2>(orig), memfun(orig.memfun) {}
    _bconnection2(TD *object, bool (TD::*memfun)(TA1, TA2))
      : _bconnection_base2<TA1, TA2>(object), memfun(memfun) {}
    ~_bconnection2() {}

    virtual _connection_base *clone()
    { return new _bconnection2<TD, TA1, TA2>(*this); }
    virtual _connection_base *duplicate(has_slots *newdest)
    { return new _bconnection2<TD, TA1, TA2>(dynamic_cast<TD *>(newdest), memfun); }

    inline bool emit(TA1 a1, TA2 a2)
    { return (dynamic_cast<TD *>(this->object)->*memfun)(a1, a2); }
  };


  template <class TA1, class TA2>
  class bsignal2 : public _signal_base
  {
  public:
    bsignal2(Lock *lock = NULL) : _signal_base(lock) {}
    ~bsignal2() {}

    template <class TD>
    void connect(TD *target, bool (TD::*memfun)(TA1, TA2))
    {
      LockBlock lock_obj(lock);
      _bconnection2<TD, TA1, TA2> *conn
	= new _bconnection2<TD, TA1, TA2>(target, memfun);
      connected_slots.push_back(conn);
      target->signal_connect(this);
    }

    inline bool operator()(TA1 a1, TA2 a2) { return emit(a1, a2); }

    bool emit(TA1 a1, TA2 a2)
    {
      LockBlock lock_obj(lock);

      list<_connection_base *>::const_iterator it = connected_slots.begin();
      list<_connection_base *>::const_iterator itEnd = connected_slots.end();
      while (it != itEnd)
	{
	  list<_connection_base *>::const_iterator itNext = it;
	  ++itNext;
	  bool result = dynamic_cast<_bconnection_base2<TA1, TA2> *>(*it)
	    ->emit(a1, a2);
	  if (result)
	    return result;
	  it = itNext;
	}

      return false;
    }
  };


  /**********************************************************************
   * Ĥbool signal/slot					*
   **********************************************************************/


  template <class TA1, class TA2, class TA3>
  class _bconnection_base3 : public _connection_base
  {
  public:
    _bconnection_base3(const _bconnection_base3 &orig)
      : _connection_base(orig) {}
    _bconnection_base3(has_slots *object) : _connection_base(object) {}
    ~_bconnection_base3() {}
    virtual bool emit(TA1 a1, TA2 a2, TA3 a3) = 0;
  };


  template <class TD, class TA1, class TA2, class TA3>
  class _bconnection3 : public _bconnection_base3<TA1, TA2, TA3>
  {
  private:
    bool (TD::*memfun)(TA1, TA2, TA3);


  public:
    _bconnection3(const _bconnection3 &orig)
      : _bconnection_base3<TA1, TA2, TA3>(orig), memfun(orig.memfun) {}
    _bconnection3(TD *object, bool (TD::*memfun)(TA1, TA2, TA3))
      : _bconnection_base3<TA1, TA2, TA3>(object), memfun(memfun) {}
    ~_bconnection3() {}

    virtual _connection_base *clone()
    { return new _bconnection3<TD, TA1, TA2, TA3>(*this); }
    virtual _connection_base *duplicate(has_slots *newdest)
    { return new _bconnection3<TD, TA1, TA2, TA3>(dynamic_cast<TD *>(newdest), memfun); }

    inline bool emit(TA1 a1, TA2 a2, TA3 a3)
    { return (dynamic_cast<TD *>(this->object)->*memfun)(a1, a2, a3); }
  };


  template <class TA1, class TA2, class TA3>
  class bsignal3 : public _signal_base
  {
  public:
    bsignal3(Lock *lock = NULL) : _signal_base(lock) {}
    ~bsignal3() {}

    template <class TD>
    void connect(TD *target, bool (TD::*memfun)(TA1, TA2, TA3))
    {
      LockBlock lock_obj(lock);
      _bconnection3<TD, TA1, TA2, TA3> *conn
	= new _bconnection3<TD, TA1, TA2, TA3>(target, memfun);
      connected_slots.push_back(conn);
      target->signal_connect(this);
    }

    inline bool operator()(TA1 a1, TA2 a2, TA3 a3) { return emit(a1, a2, a3); }

    bool emit(TA1 a1, TA2 a2, TA3 a3)
    {
      LockBlock lock_obj(lock);

      list<_connection_base *>::const_iterator it = connected_slots.begin();
      list<_connection_base *>::const_iterator itEnd = connected_slots.end();
      while (it != itEnd)
	{
	  list<_connection_base *>::const_iterator itNext = it;
	  ++itNext;
	  bool result = dynamic_cast<_bconnection_base3<TA1, TA2, TA3> *>(*it)
	    ->emit(a1, a2, a3);
	  if (result)
	    return result;
	  it = itNext;
	}

      return false;
    }
  };


  /**********************************************************************
   * ϻĤbool signal/slot					*
   **********************************************************************/


  template <class TA1, class TA2, class TA3, class TA4, class TA5, class TA6>
  class _bconnection_base6 : public _connection_base
  {
  public:
    _bconnection_base6(const _bconnection_base6 &orig)
      : _connection_base(orig) {}
    _bconnection_base6(has_slots *object) : _connection_base(object) {}
    ~_bconnection_base6() {}
    virtual bool emit(TA1 a1, TA2 a2, TA3 a3, TA4 a4, TA5 a5, TA6 a6) = 0;
  };


  template <class TD,
	    class TA1, class TA2, class TA3, class TA4, class TA5, class TA6>
  class _bconnection6 : public _bconnection_base6<TA1, TA2, TA3, TA4, TA5, TA6>
  {
  private:
    bool (TD::*memfun)(TA1, TA2, TA3, TA4, TA5, TA6);


  public:
    _bconnection6(const _bconnection6 &orig)
      : _bconnection_base6<TA1, TA2, TA3, TA4, TA5, TA6>(orig),
	memfun(orig.memfun) {}
    _bconnection6(TD *object, bool (TD::*memfun)(TA1, TA2, TA3, TA4, TA5, TA6))
      : _bconnection_base6<TA1, TA2, TA3, TA4, TA5, TA6>(object),
	memfun(memfun) {}
    ~_bconnection6() {}

    virtual _connection_base *clone()
    { return new _bconnection6<TD, TA1, TA2, TA3, TA4, TA5, TA6>(*this); }
    virtual _connection_base *duplicate(has_slots *newdest)
    { return new _bconnection6<TD, TA1, TA2, TA3, TA4, TA5, TA6>(dynamic_cast<TD *>(newdest), memfun); }

    inline bool emit(TA1 a1, TA2 a2, TA3 a3, TA4 a4, TA5 a5, TA6 a6)
    { return (dynamic_cast<TD *>(this->object)->*memfun)(a1, a2, a3, a4, a5, a6); }
  };


  template <class TA1, class TA2, class TA3, class TA4, class TA5, class TA6>
  class bsignal6 : public _signal_base
  {
  public:
    bsignal6(Lock *lock = NULL) : _signal_base(lock) {}
    ~bsignal6() {}

    template <class TD>
    void connect(TD *target, bool (TD::*memfun)(TA1, TA2, TA3, TA4, TA5, TA6))
    {
      LockBlock lock_obj(lock);
      _bconnection6<TD, TA1, TA2, TA3, TA4, TA5, TA6> *conn
	= new _bconnection6<TD, TA1, TA2, TA3, TA4, TA5, TA6>(target, memfun);
      connected_slots.push_back(conn);
      target->signal_connect(this);
    }

    inline bool operator()(TA1 a1, TA2 a2, TA3 a3, TA4 a4, TA5 a5, TA6 a6)
    { return emit(a1, a2, a3, a4, a5, a6); }

    struct EmitSignalFunc
    {
      bool &result;
      TA1 arg1;
      TA2 arg2;
      TA3 arg3;
      TA4 arg4;
      TA5 arg5;
      TA6 arg6;

      EmitSignalFunc(bool &result,
		     TA1 a1, TA2 a2, TA3 a3, TA4 a4, TA5 a5, TA6 a6)
	: result(result),
	  arg1(a1), arg2(a2), arg3(a3), arg4(a4), arg5(a5), arg6(a6) {}
      inline void operator()(_connection_base *conn)
      {
	if (!result)
	  result = dynamic_cast<_bconnection_base6<TA1, TA2, TA3, TA4, TA5, TA6> *>(conn)->emit(arg1, arg2, arg3, arg4, arg5, arg6);
      }
    };

    bool emit(TA1 a1, TA2 a2, TA3 a3, TA4 a4, TA5 a5, TA6 a6)
    {
      LockBlock lock_obj(lock);

      list<_connection_base *>::const_iterator it = connected_slots.begin();
      list<_connection_base *>::const_iterator itEnd = connected_slots.end();
      while (it != itEnd)
	{
	  list<_connection_base *>::const_iterator itNext = it;
	  ++itNext;
	  bool result = dynamic_cast<_bconnection_base6<TA1, TA2, TA3, TA4, TA5, TA6> *>(*it)->emit(a1, a2, a3, a4, a5, a6);
	  if (result)
	    return result;
	  it = itNext;
	}

      return false;
    }
  };
}
#endif

#endif /* LIBOCHUSHA_SIGSLOT_H_ */

