/*
 * Copyright (c) 2003-2008 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: hvpaned.c,v 1.10 2008/09/24 11:25:13 fuyu Exp $
 */

/* GtkPanedinternal functionȤΤǤΤޤ*/
#define GTK_COMPILATION	1

#include "config.h"

#include "libochusha/ochusha_intl.h"
#include "libochushagtk/hvpaned.h"

#include <gtk/gtk.h>
#include <glib.h>


GType
ochusha_pane_style_get_type(void)
{
  static GType pane_style_type = 0;

  if (pane_style_type == 0)
    {
      static const GEnumValue values[] =
	{
	  { OCHUSHA_HPANED_STYLE, (char *)"OCHUSHA_HPANED_STYLE", (char *)"hpaned" },
	  { OCHUSHA_VPANED_STYLE, (char *)"OCHUSHA_VPANED_STYLE", (char *)"vpaned" },
	  { 0, NULL, NULL }
	};

      pane_style_type = g_enum_register_static("OchushaPaneStyle", values);
    }

  return pane_style_type;
}


static void ochusha_hvpaned_class_init(OchushaHVPanedClass *klass);
static void ochusha_hvpaned_set_property(GObject *object, guint prop_id,
					 const GValue *value,
					 GParamSpec *pspec);
static void ochusha_hvpaned_get_property(GObject *object, guint prop_id,
					 GValue *value, GParamSpec *pspec);

static void ochusha_hvpaned_init(OchushaHVPaned *hvpaned);
static void ochusha_hvpaned_finalize(GObject *object);
static void ochusha_hvpaned_destroy(GtkObject *object);
static void ochusha_hvpaned_size_request(GtkWidget *widget,
					 GtkRequisition *requisition);
static void ochusha_hvpaned_size_allocate(GtkWidget *widget,
					  GtkAllocation *allocation);


GType
ochusha_hvpaned_get_type(void)
{
  static GType hvpaned_type = 0;

  if (hvpaned_type == 0)
    {
      static const GTypeInfo hvpaned_info =
	{
	  sizeof(OchushaHVPanedClass),
	  NULL, /* base_init */
	  NULL, /* base_finalize */
	  (GClassInitFunc)ochusha_hvpaned_class_init,
	  NULL, /* class_finalize */
	  NULL, /* class_data */
	  sizeof(OchushaHVPaned),
	  0,    /* n_preallocs */
	  (GInstanceInitFunc)ochusha_hvpaned_init,
	};

      hvpaned_type = g_type_register_static(GTK_TYPE_PANED, "OchushaHVPaned",
					    &hvpaned_info, 0);
    }

  return hvpaned_type;
}


static GtkPanedClass *parent_class = NULL;


enum
{
  PROP_0,
  PROP_PANE_STYLE,
};


static void
ochusha_hvpaned_class_init(OchushaHVPanedClass *klass)
{
  GObjectClass *o_class = G_OBJECT_CLASS(klass);
  GtkObjectClass *object_class = GTK_OBJECT_CLASS(klass);
  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(klass);

  parent_class = g_type_class_peek_parent(klass);

  /* GObject signals */
  o_class->finalize = ochusha_hvpaned_finalize;
  o_class->set_property = ochusha_hvpaned_set_property;
  o_class->get_property = ochusha_hvpaned_get_property;

  g_object_class_install_property(o_class,
				  PROP_PANE_STYLE,
				  g_param_spec_enum("pane_style",
						    PROP_("PaneStyle"),
						    PROP_("What type of pane style to be used"),
						    OCHUSHA_TYPE_PANE_STYLE,
						    OCHUSHA_HPANED_STYLE,
						    G_PARAM_READWRITE));

  /* GtkObject signals */
  object_class->destroy = ochusha_hvpaned_destroy;

  /* GtkWidget signals */
  widget_class->size_request = ochusha_hvpaned_size_request;
  widget_class->size_allocate = ochusha_hvpaned_size_allocate;
}


static void
ochusha_hvpaned_set_property(GObject *object, guint prop_id,
			     const GValue *value, GParamSpec *pspec)
{
  OchushaHVPaned *hvpaned;

  g_return_if_fail(OCHUSHA_IS_HVPANED(object));

  hvpaned = OCHUSHA_HVPANED(object);

  switch (prop_id)
    {
    case PROP_PANE_STYLE:
      ochusha_hvpaned_set_pane_style(hvpaned, g_value_get_enum(value));
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
      break;
    }
}


static void
ochusha_hvpaned_get_property(GObject *object, guint prop_id, GValue *value,
			     GParamSpec *pspec)
{
  OchushaHVPaned *hvpaned;

  g_return_if_fail(OCHUSHA_IS_HVPANED(object));

  hvpaned = OCHUSHA_HVPANED(object);

  switch (prop_id)
    {
    case PROP_PANE_STYLE:
      g_value_set_enum(value, hvpaned->pane_style);
      break;

    default:
      G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
      break;
    }
}


static void
ochusha_hvpaned_init(OchushaHVPaned *hvpaned)
{
  GtkPaned *paned;

  paned = GTK_PANED(hvpaned);
  paned->cursor_type = GDK_SB_H_DOUBLE_ARROW;
  paned->orientation = GTK_ORIENTATION_VERTICAL;

  hvpaned->pane_style = OCHUSHA_HPANED_STYLE;
}


static void
ochusha_hvpaned_finalize(GObject *object)
{
#if 0
  OchushaHVPaned *hvpaned = (OchushaHVPaned *)object;
#endif

  if (G_OBJECT_CLASS(parent_class)->finalize != NULL)
    (*G_OBJECT_CLASS(parent_class)->finalize)(object);
}


static void
ochusha_hvpaned_destroy(GtkObject *object)
{
#if 0
  OchushaHVPaned *hvpaned = (OchushaHVPaned *)object;
#endif

  if (GTK_OBJECT_CLASS(parent_class)->destroy != NULL)
    (*GTK_OBJECT_CLASS(parent_class)->destroy)(object);
}


static void
ochusha_hvpaned_size_request(GtkWidget *widget, GtkRequisition *requisition)
{
  OchushaHVPaned *hvpaned = OCHUSHA_HVPANED(widget);
  GtkPaned *paned = GTK_PANED(widget);
  GtkRequisition child_requisition;

  requisition->width = 0;
  requisition->height = 0;

  if (paned->child1 && GTK_WIDGET_VISIBLE(paned->child1))
    {
      gtk_widget_size_request(paned->child1, &child_requisition);

      requisition->height = child_requisition.height;
      requisition->width = child_requisition.width;
    }

  if (paned->child2 && GTK_WIDGET_VISIBLE (paned->child2))
    {
      gtk_widget_size_request(paned->child2, &child_requisition);

      if (hvpaned->pane_style == OCHUSHA_HPANED_STYLE)
	{
	  requisition->height = MAX(requisition->height,
				    child_requisition.height);
	  requisition->width += child_requisition.width;
	}
      else
	{
	  requisition->width = MAX(requisition->width,
				   child_requisition.width);
	  requisition->height += child_requisition.height;
	}
    }

  requisition->height += GTK_CONTAINER(paned)->border_width * 2;
  requisition->width += GTK_CONTAINER(paned)->border_width * 2;
  
  if (paned->child1 && GTK_WIDGET_VISIBLE(paned->child1) &&
      paned->child2 && GTK_WIDGET_VISIBLE(paned->child2))
    {
      int handle_size;

      gtk_widget_style_get(widget, "handle-size", &handle_size, NULL);
      if (hvpaned->pane_style == OCHUSHA_HPANED_STYLE)
	requisition->width += handle_size;
      else
	requisition->height += handle_size;
    }
}


static void
ochusha_hvpaned_size_allocate(GtkWidget *widget, GtkAllocation *allocation)
{
  OchushaHVPaned *hvpaned = OCHUSHA_HVPANED(widget);
  GtkPaned *paned = GTK_PANED (widget);
  int border_width = GTK_CONTAINER (paned)->border_width;

  widget->allocation = *allocation;

  if (paned->child1 && GTK_WIDGET_VISIBLE(paned->child1) &&
      paned->child2 && GTK_WIDGET_VISIBLE(paned->child2))
    {
      GtkAllocation child1_allocation;
      GtkAllocation child2_allocation;
      GtkRequisition child1_requisition;
      GtkRequisition child2_requisition;
      int handle_size;
      
      gtk_widget_style_get (widget, "handle-size", &handle_size, NULL);

      gtk_widget_get_child_requisition(paned->child1, &child1_requisition);
      gtk_widget_get_child_requisition(paned->child2, &child2_requisition);

      if (hvpaned->pane_style == OCHUSHA_HPANED_STYLE)
	{
	  gtk_paned_compute_position(paned,
				     MAX(1, widget->allocation.width
					 - handle_size
					 - 2 * border_width),
				     child1_requisition.width,
				     child2_requisition.width);

	  paned->handle_pos.x = widget->allocation.x + paned->child1_size + border_width;
	  paned->handle_pos.y = widget->allocation.y + border_width;
	  paned->handle_pos.width = handle_size;
	  paned->handle_pos.height = MAX(1, widget->allocation.height - 2 * border_width);
	}
      else
	{
	  gtk_paned_compute_position(paned,
				     MAX(1, widget->allocation.height
					 - handle_size
					 - 2 * border_width),
				     child1_requisition.height,
				     child2_requisition.height);

	  paned->handle_pos.x = widget->allocation.x + border_width;
	  paned->handle_pos.y = widget->allocation.y + paned->child1_size + border_width;
	  paned->handle_pos.width = MAX(1, (int) widget->allocation.width - 2 * border_width);
	  paned->handle_pos.height = handle_size;
	}
      
      
      if (GTK_WIDGET_REALIZED(widget))
	{
	  if (GTK_WIDGET_MAPPED(widget))
	    gdk_window_show(paned->handle);
	  if (hvpaned->pane_style == OCHUSHA_HPANED_STYLE)
	    {
	      gdk_window_move_resize(paned->handle,
				     paned->handle_pos.x,
				     paned->handle_pos.y,
				     handle_size,
				     paned->handle_pos.height);
	    }
	  else
	    {
	      gdk_window_move_resize(paned->handle,
				     paned->handle_pos.x,
				     paned->handle_pos.y,
				     paned->handle_pos.width,
				     handle_size);
	    }
	}

      if (hvpaned->pane_style == OCHUSHA_HPANED_STYLE)
	{
	  child1_allocation.height = child2_allocation.height = MAX(1, (int) allocation->height - border_width * 2);
	  child1_allocation.width = paned->child1_size;
	  child1_allocation.x = widget->allocation.x + border_width;
	  child1_allocation.y = child2_allocation.y = widget->allocation.y + border_width;

	  child2_allocation.x = child1_allocation.x + child1_allocation.width +  paned->handle_pos.width;
	  child2_allocation.width = MAX(1, widget->allocation.x + widget->allocation.width - child2_allocation.x - border_width);
	}
      else
	{
	  child1_allocation.width = child2_allocation.width = MAX(1, (int) allocation->width - border_width * 2);
	  child1_allocation.height = paned->child1_size;
	  child1_allocation.x = child2_allocation.x = widget->allocation.x + border_width;
	  child1_allocation.y = widget->allocation.y + border_width;

	  child2_allocation.y = child1_allocation.y + child1_allocation.height + paned->handle_pos.height;
	  child2_allocation.height = MAX(1, widget->allocation.y + widget->allocation.height - child2_allocation.y - border_width);
	}
      
      
      /* Now allocate the childen, making sure, when resizing not to
       * overlap the windows
       */
      if (hvpaned->pane_style == OCHUSHA_HPANED_STYLE)
	{
	  if (GTK_WIDGET_MAPPED(widget) &&
	      paned->child1->allocation.width < child1_allocation.width)
	    {
	      gtk_widget_size_allocate(paned->child2, &child2_allocation);
	      gtk_widget_size_allocate(paned->child1, &child1_allocation);
	    }
	  else
	    {
	      gtk_widget_size_allocate(paned->child1, &child1_allocation);
	      gtk_widget_size_allocate(paned->child2, &child2_allocation);
	    }
	}
      else
	{
	  if (GTK_WIDGET_MAPPED(widget) &&
	      paned->child1->allocation.height < child1_allocation.height)
	    {
	      gtk_widget_size_allocate(paned->child2, &child2_allocation);
	      gtk_widget_size_allocate(paned->child1, &child1_allocation);
	    }
	  else
	    {
	      gtk_widget_size_allocate(paned->child1, &child1_allocation);
	      gtk_widget_size_allocate(paned->child2, &child2_allocation);
	    }
	}
      hvpaned->had_child2 = TRUE;
    }
  else
    {
      GtkAllocation child_allocation;

      if (GTK_WIDGET_REALIZED (widget))      
	gdk_window_hide(paned->handle);
	  
      child_allocation.x = widget->allocation.x + border_width;
      child_allocation.y = widget->allocation.y + border_width;
      child_allocation.width = MAX(1, allocation->width - 2 * border_width);
      child_allocation.height = MAX(1, allocation->height - 2 * border_width);

      if (paned->child1 && GTK_WIDGET_VISIBLE(paned->child1))
	{
	  gtk_widget_size_allocate(paned->child1, &child_allocation);
	  if (hvpaned->had_child2)
	    {
	      hvpaned->had_child2 = FALSE;
	      if (gtk_paned_get_position(paned) == 0)
		{
		  gtk_widget_unmap(paned->child1);
		  gtk_widget_set_child_visible(paned->child1, TRUE);
		  gtk_widget_map(paned->child1);
		}
	    }
	}
      else if (paned->child2 && GTK_WIDGET_VISIBLE(paned->child2))
	{
	  gtk_widget_size_allocate(paned->child2, &child_allocation);
	  hvpaned->had_child2 = TRUE;
	}
    }
}


GtkWidget *
ochusha_hvpaned_new(OchushaPaneStyleType style)
{
  return g_object_new(OCHUSHA_TYPE_HVPANED,
		      "pane_style", style,
		      NULL);
}


void
ochusha_hvpaned_set_pane_style(OchushaHVPaned *hvpaned,
			       OchushaPaneStyleType style)
{
  g_return_if_fail(OCHUSHA_IS_HVPANED(hvpaned));

  if (hvpaned->pane_style != style)
    {
      GtkPaned *paned = GTK_PANED(hvpaned);
      hvpaned->pane_style = style;

      if (hvpaned->pane_style == OCHUSHA_HPANED_STYLE)
	{
	  paned->cursor_type = GDK_SB_H_DOUBLE_ARROW;
	  paned->orientation = GTK_ORIENTATION_VERTICAL;
	}
      else
	{
	  paned->cursor_type = GDK_SB_V_DOUBLE_ARROW;
	  paned->orientation = GTK_ORIENTATION_HORIZONTAL;
	}

      if (GDK_IS_WINDOW(paned->handle))
	{
	  GdkCursor *cursor = gdk_cursor_new_for_display(gtk_widget_get_display(GTK_WIDGET(paned)), paned->cursor_type);
	  gdk_window_set_cursor(paned->handle, cursor);
	  gdk_cursor_unref(cursor);
	}

      if (GTK_WIDGET_VISIBLE(hvpaned))
	gtk_widget_queue_resize(GTK_WIDGET(hvpaned));
    }

  g_object_notify(G_OBJECT(hvpaned), "pane_style");
}


OchushaPaneStyleType
ochusha_hvpaned_get_pane_style(OchushaHVPaned *hvpaned)
{
  g_return_val_if_fail(OCHUSHA_IS_HVPANED(hvpaned), 0);
  return hvpaned->pane_style;
}

