/*
 * Copyright (c) 2003-2008 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: misc_utils.c,v 1.13 2008/10/19 17:53:15 itn2002 Exp $
 */

#include "config.h"

#include "libochushagtk/misc_utils.h"

#include <glib.h>
#include <gtk/gtk.h>

#include <stdio.h>


/*
 * GTK+-2.4ǤGtkItemFactorydeprecateƤޤä
 * GtkUIManagerѤAPIǤϥݥåץåץ˥塼ɽΤ
 * ݤˤʤäƤ롣
 * ʤΤǡgtk_item_factory_popup_with_dataؿȤ
 */
static GQuark popup_menu_target_id;


static void
popup_delete_data(GObject *object, GtkUIManager *manager)
{
  g_signal_handlers_disconnect_by_func(object,
				       popup_delete_data,
				       manager);
  g_object_set_qdata(G_OBJECT(manager), popup_menu_target_id, NULL);
}


void
ochusha_ui_manager_popup_with_data(GtkUIManager *ui_manager,
				   const char *menu_path,
				   gpointer popup_data,
				   GDestroyNotify destroy,
				   GtkMenuPositionFunc position_func,
				   gpointer position_data,
				   guint button,
				   guint32 event_time)
{
  static gboolean initialized = FALSE;
  GtkWidget *tmp_widget;
  GtkMenu *menu;

  if (!initialized)
    {
      popup_menu_target_id
	= g_quark_from_static_string("MiscUtils::PopupMenuTarget");
      
      initialized = TRUE;
    }

  tmp_widget = gtk_ui_manager_get_widget(ui_manager, menu_path);
  g_return_if_fail(GTK_IS_MENU(tmp_widget));

  menu = GTK_MENU(tmp_widget);
  g_object_set_qdata_full(G_OBJECT(ui_manager), popup_menu_target_id,
			  popup_data, destroy);
  g_signal_connect(menu, "selection-done",
		   G_CALLBACK(popup_delete_data), ui_manager);
  gtk_menu_popup(menu, NULL, NULL, position_func, position_data,
		 button, event_time);
}


gpointer
ochusha_ui_manager_get_popup_data(GtkUIManager *ui_manager)
{
  return g_object_get_qdata(G_OBJECT(ui_manager), popup_menu_target_id);
}


static void
check_children(GtkWidget *widget, gpointer data)
{
  GtkWidget **result = (GtkWidget **)data;
  if (*result != NULL)
    return;

  if (gtk_widget_is_focus(widget))
    {
      *result = widget;
      return;
    }

  if (GTK_IS_CONTAINER(widget) && ochusha_find_focus_widget(widget) != NULL)
    *result = widget;
}


/*
 * Ϳ줿widgetλwidget˸focus򰮤äƤwidget
 * ä餽widgetؤΥݥ󥿤֤
 */
GtkWidget *
ochusha_find_focus_widget(GtkWidget *widget)
{
  if (widget == NULL)
    return NULL;

  if (gtk_widget_is_focus(widget))
    return widget;

  if (GTK_IS_CONTAINER(widget))
    {
      GtkWidget *focus = NULL;
      gtk_container_forall(GTK_CONTAINER(widget), check_children, &focus);
      if (focus != NULL)
	return focus;
    }
  return NULL;
}


static GdkAtom text_uri_list_atom = NULL;


static void
init_atoms()
{
  if (text_uri_list_atom != NULL)
    return;

#if GTK_CHECK_VERSION(2,10,0)
  text_uri_list_atom = gdk_atom_intern_static_string("text/uri-list");
#else
  text_uri_list_atom = gdk_atom_intern("text/uri-list", FALSE);
#endif
}


/*
 * ĤAPIäơʤ̵
 */
gchar **
ochusha_clipboard_wait_for_uri(GtkClipboard *clipboard)
{
  GtkSelectionData *data;
  gchar **uri_list;

  g_return_val_if_fail(GTK_IS_CLIPBOARD(clipboard), NULL);

  init_atoms();

  data = gtk_clipboard_wait_for_contents(clipboard, text_uri_list_atom);
  if (data == NULL)
    return NULL;

  uri_list = gtk_selection_data_get_uris(data);

  gtk_selection_data_free(data);

  return uri_list;
}


gboolean
ochusha_clipboard_wait_is_uri_available(GtkClipboard *clipboard)
{
  g_return_val_if_fail(GTK_IS_CLIPBOARD(clipboard), FALSE);

  init_atoms();

  return gtk_clipboard_wait_is_target_available(clipboard, text_uri_list_atom);
}
