/* -*- c++ -*-
 * Copyright (c) 2003-2008 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: symbol_table.h,v 1.10 2008/09/24 11:26:46 fuyu Exp $
 */

#ifndef _LIBTSCORE_SYMBOL_TABLE_H_
#define _LIBTSCORE_SYMBOL_TABLE_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libtscore/cell.h"


using namespace std;
namespace LibTSCore
{
  /*
   * Symbol table manages a table whose contents are essentially
   * interned strings.
   *
   * This doesn't use any kind of weak reference techniques at this
   * time, and thus, all symbols will be alive until the end of this
   * object's life.
   */
  class TSCore;
  class SymbolTable
  {
  private:
    TSCore &core;
    Cell *table;

    Cell *insert0(Cell *name);

  public:
    SymbolTable(TSCore &tscore);
    ~SymbolTable();

    void purge_all();

    Cell *lookup(Cell *symbol) const;
    Cell *lookup(const char *name) const;
    Cell *insert(Cell *name);
    Cell *insert(const char *name);
    Cell *list_all_symbols() const;
  };
}

#endif

#endif	/* _LIBTSCORE_SYMBOL_TABLE_H_ */
