/* -*- c++ -*-
 * Copyright (c) 2003-2008 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: bbs_thread_view.h,v 1.26 2008/10/16 13:29:27 fuyu Exp $
 */

#ifndef _BBS_THREAD_VIEW_H_
#define _BBS_THREAD_VIEW_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "bbs_thread_renderer.h"
#include "bbs_thread_ui.h"

#include "selection_utils.h"

#include "libochusha/bbs_thread.h"
#include "libochusha/html_utils.h"
#include "libochusha/regexp.h"
#include "libochusha/sigslot.h"

#include "libochushagtk/hypertext_buffer.h"
#include "libochushagtk/hypertext_view.h"

#include <list>
#include <vector>

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{
  class BBSThreadPopupView;
  class Controller;
  class ResponseEditorWindow;
  class TabLabel;


  class BBSThreadViewSelectionData : public SelectionData
  {
  private:
    BBSThread *thread;
    gchar *link;
    char *canonical_link;


  protected:
    virtual const GtkTargetEntry *get_targets();
    virtual guint get_n_targets();

    virtual void get_data(GtkClipboard *clipboard, GtkSelectionData *data,
			  guint info);
    // 줬true֤顢delete this¹Ԥ롣
    virtual bool clear_data(GtkClipboard *clipboard);


  public:
    BBSThreadViewSelectionData(BBSThread *thread);
    virtual ~BBSThreadViewSelectionData();

    void set_link(const gchar *value);
    const char *get_canonical_link();

    inline const gchar *get_link() const { return link; }
  };


  struct ResponseRecord
  {
    gint offset;
    size_t length;
    ResponseRecord() : offset(0), length(0) {}
    ResponseRecord(int offset, size_t length)
      : offset(offset), length(length) {}
    ResponseRecord(const ResponseRecord &record)
      : offset(record.offset), length(record.length) {}
  };


  class BBSThreadViewBase : public has_slots
  {
  protected:
    Controller &controller;
    smart_ptr<BBSThread> thread;

    OchushaHypertextBuffer *buffer;
    GtkWidget *widget;
    OchushaHypertextView *view;

    BBSThreadRenderer renderer;


  private:
    guint message_id;
    guint selection_done_id;

    bool popup_menu;

    BBSThreadViewSelectionData selection_data;

    gchar *popup_link;
    BBSThreadPopupView *popup_view;

    static void widget_destroy_cb(GtkWidget *widget, BBSThreadViewBase *view);
    static void widget_unmap_cb(GtkWidget *widget, BBSThreadViewBase *view);

    static void popup_destroy_cb(GtkWidget *widget, BBSThreadViewBase *view);
    static gboolean link_mouse_over_cb(OchushaHypertextView *widget,
				       GdkEventMotion *event,
				       const gchar *link,
				       BBSThreadViewBase *view);
    static gboolean link_mouse_out_cb(OchushaHypertextView *widget,
				      GdkEventMotion *event,
				      BBSThreadViewBase *view);
    static void populate_popup_cb(GtkTextView *text_view, GtkMenu *popup_menu,
				  BBSThreadViewBase *view);
    static void selection_done_cb(GtkMenu *popup_menu, BBSThreadViewBase *view);
    static void popup_menu_destroy_cb(BBSThreadViewBase *view);
    static gboolean link_mouse_press_cb(OchushaHypertextView *widget,
					GdkEventButton *event,
					const gchar *link,
					BBSThreadViewBase *view);
    static gboolean link_mouse_release_cb(OchushaHypertextView *widget,
					  GdkEventButton *event,
					  const gchar *link,
					  BBSThreadViewBase *view);
    static void copy_clipboard_cb(OchushaHypertextView *widget,
				  BBSThreadViewBase *view);


  protected:
    BBSThreadViewBase(Controller &controller, BBSThread *thread);
    virtual ~BBSThreadViewBase();

    virtual ResponseCursor *get_response_cursor() = 0;
    virtual GtkWindow *get_window() = 0;
    inline bool has_popup() const { return popup_view != NULL; }
    inline bool has_popup_menu() const { return popup_menu; }

    virtual void show() = 0;
    virtual void hide(bool now) = 0;


  public:
    inline BBSThread *get_thread() { return thread; }
    inline const char *get_last_event_link() const
    { return selection_data.get_link(); }
    static BBSThreadViewBase *get_view(GtkWidget *widget);
  };


  class BBSThreadPopupView : public BBSThreadViewBase
  {
  private:
    smart_ptr<ResponseCursor> response_cursor;

    GtkWidget *window;

    guint popup_id;
    guint popdown_id;

    bool pointed;

    void initialize_common(GtkWindow *parent);

    static gboolean enter_notify_event_cb(GtkWidget *widget,
					  GdkEventCrossing *event,
					  BBSThreadPopupView *view);
    static gboolean leave_notify_event_cb(GtkWidget *widget,
					  GdkEventCrossing *event,
					  BBSThreadPopupView *view);
    static void size_request_cb(GtkWidget *widget, GtkRequisition *requisition,
				BBSThreadPopupView *view);

    static gboolean show_now(BBSThreadPopupView *view);
    static gboolean hide_now(BBSThreadPopupView *view);


  protected:
    virtual ~BBSThreadPopupView();

    virtual ResponseCursor *get_response_cursor();
    virtual GtkWindow *get_window();

    virtual void show();
    virtual void hide(bool now);


  public:
    BBSThreadPopupView(Controller &controller, BBSThread *thread,
		       GtkWindow *parent,
		       ResponseCursor *cursor, guint from, guint to);
    BBSThreadPopupView(Controller &controller, BBSThread *thread,
		       GtkWindow *parent,
		       ResponseCursor *cursor, const gchar *id);
    BBSThreadPopupView(Controller &controller, BBSThread *thread,
		       GtkWindow *parent,
		       GtkWidget *image, const gchar *link);
    BBSThreadPopupView(Controller &controller, BBSThread *thread,
		       GtkWindow *parent,
		       const gchar *title);
  };


  class BBSThreadView : public BBSThreadViewBase
  {
  private:
    smart_ptr<ResponseCursor> response_cursor;

    guint number_of_responses_shown;
    guint append_responses_id;

    vector<ResponseRecord> records;

    guint initial_view_position;
    GtkWidget *my_own_bookmark;
    guint my_own_bookmark_position;

    list<gint> backward_history;
    list<gint> forward_history;

    GtkWidget *post_window;
    guint post_window_destroy_id;

    GtkWidget *tab_label;
    GdkColor *tab_label_normal_color;
    guint auto_refresher_id;
    guint previous_interval;
    int previous_number_of_responses_got;
    bool enable_transparent_a_bone;
    bool disable_a_bone;
    bool enable_chain_a_bone;


    BBSThreadView(Controller &controller, BBSThread *thread);

    virtual ~BBSThreadView();

    virtual ResponseCursor *get_response_cursor();
    virtual GtkWindow *get_window();

    virtual void show();
    virtual void hide(bool now);


    // 鲶ѤϢ

    // OchushaHypertextBuffer/Viewoffset֤widget
    // records򹹿
    void insert_widget_at_offset(GtkWidget *widget, gint offset);

    // OchushaHypertextBuffer/Viewwidgetrecords
    // 
    void remove_widget(GtkWidget *widget);

    GtkWidget *create_bookmark();
    static void bookmark_clicked_cb(GtkWidget *widget, BBSThreadView *view);

    // ޤǲѤϢ


    void append_response(guint res_num, Response &response);
    static gboolean append_responses_on_idle(BBSThreadView *view);
    void state_changed_cb();


    static void post_window_destroy_cb(GtkWidget *widget, BBSThreadView *view);
    static gboolean auto_refresher_timeout_cb(BBSThreadView *view);


  public:
    // BBSThreadView֥Ȥget_widget()֤GtkWidgetȰϡ
    static GtkWidget *create(Controller &controller, BBSThread *thread,
			     TabLabel *label, int initial_pos);
    static void update(GtkWidget *widget, TabLabel *label, bool automatic);
    static BBSThreadView *get_view(GtkWidget *widget);

    void show_post_window();
    ResponseEditorWindow *get_post_window();
    void jump_to_bookmark();
    void goto_bottom();
    void goto_top();
    void goto_next();
    void goto_prev();

    // 쥹ֹres_numΥ쥹ľ˲ѤΤ
    void insert_my_own_bookmark(guint res_num);

    void specify_view_position(int position);
    bool bookmark_exist() { return my_own_bookmark != NULL; }
    bool forward_history_exist() { return !forward_history.empty(); }
    bool backward_history_exist() { return !backward_history.empty(); }

    void remember_current_view_position();
  };


}


#endif	/* __cplusplus */

#endif	/* _BBS_THREAD_VIEW_H_ */
