/* -*- c++ -*-
 * Copyright (c) 2003-2008 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: main_window.h,v 1.20 2008/10/20 15:57:38 fuyu Exp $
 */

#ifndef _MAIN_WINDOW_H_
#define _MAIN_WINDOW_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/network_agent.h"
#include "libochusha/sigslot.h"

#include "bbs_thread_ui.h"
#include "board_ui.h"
#include "boardlist_ui.h"
#include "bookmarks_ui.h"
#include "threadlist_ui.h"
#include "view_manager.h"


#include <list>

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{
  class Controller;

  struct AccessMessage
  {
    char *url;
    guint message_id;

    AccessMessage(const char *url, guint message_id);
    AccessMessage(const AccessMessage &orig);
    ~AccessMessage();
  };


  class MainWindow : public has_slots
  {
    friend class CommandPipe;
    friend class Controller;

  private:
    Controller &controller;

    GtkActionGroup *action_group;

    GtkWindow *toplevel;
    GtkBox *main_box;			// GtkVBox

    GtkWidget *menubar;
    GtkWidget *navigation_toolbar;


    // ɥС()ڥȥڥʥƥ
    GtkWidget *main_container;		// GtkHPaned


    // ɥС
    ModNotebook *sidebar_book;
    GtkWidget *boardlist_view;
    GtkWidget *bookmarks_view;
    GtkWidget *history_view;		// ͽ
    GtkWidget *post_history_view;	// ͽ

#define BOARDLIST_SIDEBAR	(1 << 0)
#define BOOKMARKS_SIDEBAR	(1 << 1)
#define HISTORY_SIDEBAR		(1 << 2)
#define POST_HISTORY_SIDEBAR	(1 << 3)
#define SHOWN_SIDEBAR_MASK	(BOARDLIST_SIDEBAR | BOOKMARKS_SIDEBAR \
				 | HISTORY_SIDEBAR | POST_HISTORY_SIDEBAR)
    unsigned int shown_sidebar_items;


    // ()ڥȥڥƤĤˤޤ
    ViewManager view_manager;


    // СϢ
    GtkWidget *searchbar;		// OchushaSearchbar


    // ơС
    GtkBox *status_box;			// GtkHBox
    GtkStatusbar *statusbar;
    GtkWidget *connection_image;	// GtkImage
    GtkWidget *edit_image;		// GtkImage


    // statusbarcontext ID
    guint searchbar_context_id;		// Сcontext
    guint boardlist_context_id;		// İcontext
    guint link_context_id;		// ɽcontext
    guint network_context_id;		// ͥåȥcontext

    // ͥåȥ֤ΥơСɽ
    list<AccessMessage> access_message_list;

    bool maximized;

    GtkActionGroup *create_action_group();
    void setup_main_ui();


    static void notify_position_cb(GtkPaned *paned, GParamSpec *arg1,
				   MainWindow *window);
    static void notify_position_set_cb(GtkPaned *paned, GParamSpec *arg1,
				       MainWindow *window);
    static void match_case_toggled_cb(GtkToggleButton *button,
				      MainWindow *window);
    static void use_regexp_toggled_cb(GtkToggleButton *button,
				      MainWindow *window);
    static void delete_event_cb(GtkWidget *widget, GdkEventAny *event,
				MainWindow *window);
    static gboolean window_state_event_cb(GtkWidget *widget,
					  GdkEventWindowState *event,
					  MainWindow *window);

    static void show_navigation_toolbar_toggled_cb(GtkToggleAction *action,
						   MainWindow *window);

    static gboolean connection_image_pressed_cb(GtkWidget *event_box,
						GdkEventButton *event,
						Controller *controller);
    static void work_offline_toggled_cb(GtkToggleAction *action,
					MainWindow *window);

    static gboolean edit_image_pressed_cb(GtkWidget *event_box,
					  GdkEventButton *event,
					  GtkToggleAction *action);
    static void embeded_response_editor_toggled_cb(GtkToggleAction *action,
						   MainWindow *window);


    // ɥСϢ
    static gboolean sidebar_button_press_event_cb(ModNotebook *notebook,
						  GdkEventButton *event,
						  MainWindow *window);
    static void show_boardlist_sidebar_toggled_cb(GtkToggleAction *action,
						  MainWindow *window);
    static void show_bookmarks_sidebar_toggled_cb(GtkToggleAction *action,
						  MainWindow *window);
    static void show_history_sidebar_toggled_cb(GtkToggleAction *action,
						MainWindow *window);
    static void show_post_history_sidebar_toggled_cb(GtkToggleAction *action,
						     MainWindow *window);

    void update_sidebar_view();


    // ͥåȥsigslotʥϥɥ
    void notify_access_status_cb(const char *url, NetworkAccessStatus status);


  public:
    MainWindow(Controller &controller);
    ~MainWindow();

    void show();
    void save_states();
    void prepare_session();
    void restore_session();

    void prepare_shutdown();
  };


}


#endif	/* __cplusplus */

#endif	/* _MAIN_WINDOW_H_ */
