/* -*- c++ -*-
 * Copyright (c) 2003-2008 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: selection_utils.h,v 1.3 2008/09/24 11:56:59 fuyu Exp $
 */

#ifndef _SELECTION_UTILS_H_
#define _SELECTION_UTILS_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include <vector>

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{


  class SelectionDataProxy;

  class SelectionData
  {
    friend class SelectionDataProxy;

  private:
    static void get_data(GtkClipboard *clipboard,
			 GtkSelectionData *selection_data,
			 guint info, gpointer data);
    static void clear_data(GtkClipboard *clipboard, gpointer data);


  protected:
    virtual const GtkTargetEntry *get_targets() = 0;
    virtual guint get_n_targets() = 0;

    virtual void get_data(GtkClipboard *clipboard, GtkSelectionData *data,
			  guint info) = 0;
    // 줬true֤顢delete this¹Ԥ롣
    virtual bool clear_data(GtkClipboard *clipboard) = 0;


  public:
    virtual ~SelectionData();

    gboolean set_to(GtkClipboard *clipboard);

    static gboolean set_uris(GtkSelectionData *data,
			     const vector<const char *> &uris);
    static gboolean set_uris(GtkSelectionData *data,
			     const vector<char *> &uris);
  };


  class TextBufferSelectionData : public SelectionData
  {
  private:
    GtkTextBuffer *buffer;


  protected:
    virtual const GtkTargetEntry *get_targets();
    virtual guint get_n_targets();

    virtual void get_data(GtkClipboard *clipboard, GtkSelectionData *data,
			  guint info);
    // 줬true֤顢delete this¹Ԥ롣
    virtual bool clear_data(GtkClipboard *clipboard);


  public:
    TextBufferSelectionData(GtkTextBuffer *buffer);
    virtual ~TextBufferSelectionData();
  };


  class SelectionDataProxy : public SelectionData
  {
  private:
    SelectionData *data;
    GtkWidget *widget;
    guint destroy_id;

    static void destroy_cb(GtkWidget *widget, SelectionDataProxy *proxy);


  protected:
    virtual const GtkTargetEntry *get_targets();
    virtual guint get_n_targets();

    virtual void get_data(GtkClipboard *clipboard, GtkSelectionData *data,
			  guint info);
    // 줬true֤顢delete this¹Ԥ롣
    virtual bool clear_data(GtkClipboard *clipboard);


  public:
    SelectionDataProxy(SelectionData *data, GtkWidget *widget);
    virtual ~SelectionDataProxy();
  };


}


#endif	/* __cplusplus */

#endif	/* _SELECTION_UTILS_H_ */
