/* -*- c++ -*-
 * Copyright (c) 2003-2008 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: tab_label.cc,v 1.19 2008/10/25 08:45:56 fuyu Exp $
 */

#include "config.h"

#include "libochusha/ochusha_intl.h"

#if defined(__GNUG__) && !defined(__ICC)
#pragma implementation
#endif
#include "tab_label.h"

#include "controller.h"

#include "libochushagtk/icon_label.h"
#include "libochushagtk/stock.h"

#include "libochushagtk_lgpl/mod_pixbuf_simple_anim.h"

#include <gtk/gtk.h>

#include <cstring>


namespace Ochusha
{

#undef VERBOSE_DESTRUCTOR
#define ICON_THEME_TEST		1
#define TAB_LABEL_ICON_SIZE	16

// GtkWidgetTabLabel֥ȤФȤ˻Ȥ
static GQuark label_id = g_quark_from_static_string("TabLabel::label");


class TabLabelIconSet
{
private:
  gchar *stock_id;

  GdkPixbufAnimation *wait_animation;
  GdkPixbufAnimation *preparation_animation;
  GdkPixbufAnimation *progress_animation;
  GdkPixbuf *done_icon;
  GdkPixbuf *error_icon;
  GdkPixbuf *notice_icon;

  void composite_stock_with_addons(TabLabelManager &manager);


public:
  TabLabelIconSet(const gchar *stock_id, TabLabelManager &manager);
  ~TabLabelIconSet();

  inline const gchar *get_stock_id() const { return stock_id; }
  inline GdkPixbufAnimation *get_wait_animation() const
  { return wait_animation; }
  inline GdkPixbufAnimation *get_preparation_animation() const
  { return preparation_animation; }
  inline GdkPixbufAnimation *get_progress_animation() const
  { return progress_animation; }
  inline GdkPixbuf *get_done_icon() const { return done_icon; }
  inline GdkPixbuf *get_error_icon() const { return error_icon; }
  inline GdkPixbuf *get_notice_icon() const { return notice_icon; }
};


TabLabelIconSet::TabLabelIconSet(const gchar *stock_id,
				 TabLabelManager &manager)
  : stock_id(g_strdup(stock_id)),
    wait_animation(NULL),
    preparation_animation(NULL),
    progress_animation(NULL),
    done_icon(NULL),
    error_icon(NULL),
    notice_icon(NULL)
{
  composite_stock_with_addons(manager);
}


TabLabelIconSet::~TabLabelIconSet()
{
  g_free(stock_id);
  if (wait_animation != NULL)
    g_object_unref(wait_animation);

  if (preparation_animation != NULL)
    g_object_unref(preparation_animation);

  if (progress_animation != NULL)
    g_object_unref(progress_animation);

  if (done_icon != NULL)
    g_object_unref(done_icon);

  if (error_icon != NULL)
    g_object_unref(error_icon);

  if (notice_icon != NULL)
    g_object_unref(notice_icon);
}


static GdkPixbuf *
composite_pixbufs(GdkPixbuf *base, GdkPixbuf *addon)
{
  if (base == NULL)
    {
      if (addon != NULL)
	g_object_ref(addon);
      g_return_val_if_fail(base != NULL, addon);	// ɬ
    }

  if (addon == NULL)
    {
      g_object_ref(base);
      g_return_val_if_fail(addon != NULL, base);	// ɬ
    }

  if (gdk_pixbuf_get_colorspace(base) != GDK_COLORSPACE_RGB
      || gdk_pixbuf_get_bits_per_sample(base) != 8
      || !gdk_pixbuf_get_has_alpha(base)
      || gdk_pixbuf_get_n_channels(base) != 4)
    {
      fprintf(stderr, "Unsupported type of GdkPixbuf given as the base!\n");
      g_object_ref(base);
      return base;
    }

  if (gdk_pixbuf_get_colorspace(addon) != GDK_COLORSPACE_RGB
      || gdk_pixbuf_get_bits_per_sample(addon) != 8
      || !gdk_pixbuf_get_has_alpha(addon)
      || gdk_pixbuf_get_n_channels(addon) != 4)
    {
      fprintf(stderr, "Unsupported type of GdkPixbuf given as the addon!\n");
      g_object_ref(base);
      return base;
    }

  GdkPixbuf *result_pixbuf = gdk_pixbuf_copy(base);
  if (result_pixbuf == NULL)
    {
      if (base != NULL)
	g_object_ref(base);
      fprintf(stderr, "Out of memory");
      return base;
    }

  gint base_width = gdk_pixbuf_get_width(result_pixbuf);
  gint base_height = gdk_pixbuf_get_height(result_pixbuf);
  gint addon_width = gdk_pixbuf_get_width(addon);
  gint addon_height = gdk_pixbuf_get_height(addon);

  int base_rowstride = gdk_pixbuf_get_rowstride(result_pixbuf);
  guchar *base_pixels = gdk_pixbuf_get_pixels(result_pixbuf);

  int addon_rowstride = gdk_pixbuf_get_rowstride(addon);
  guchar *addon_pixels = gdk_pixbuf_get_pixels(addon);

  for (int y = 0; y < base_height && y < addon_height; y++)
    {
      for (int x = 0; x < base_width && x < addon_width; x++)
	{
	  /* 4 is the constant for n_channels we support */
	  guchar *addon_pixel = addon_pixels + y * addon_rowstride + x * 4;
	  guchar addon_r = addon_pixel[0];
	  guchar addon_g = addon_pixel[1];
	  guchar addon_b = addon_pixel[2];
	  guchar addon_a = addon_pixel[3];

	  guchar *base_pixel = base_pixels + y * base_rowstride + x * 4;
	  guchar base_r = base_pixel[0];
	  guchar base_g = base_pixel[1];
	  guchar base_b = base_pixel[2];
	  guchar base_a = base_pixel[3];

	  guchar base_o = 255 - addon_a;
	  base_pixel[0]
	    = ((base_r * base_o) / 256) + ((addon_r * addon_a) / 256);
	  base_pixel[1]
	    = ((base_g * base_o) / 256) + ((addon_g * addon_a) / 256);
	  base_pixel[2]
	    = ((base_b * base_o) / 256) + ((addon_b * addon_a) / 256);
	  base_pixel[3] = base_a > addon_a ? base_a : addon_a;
	}
    }

  return result_pixbuf;
}


void
TabLabelIconSet::composite_stock_with_addons(TabLabelManager &manager)
{
  GtkIconTheme *icon_theme = NULL;

  GtkWindow *toplevel = manager.controller.get_toplevel();
  if (toplevel != NULL)
    {
      GtkWidget *widget = GTK_WIDGET(toplevel);

      GdkScreen *screen;
      if (gtk_widget_has_screen(widget))
	screen = gtk_widget_get_screen(widget);
      else
	screen = gdk_screen_get_default();

      icon_theme = gtk_icon_theme_get_for_screen(screen);
    }

  if (icon_theme == NULL)
    icon_theme = gtk_icon_theme_get_default();

  GdkPixbuf *base = gtk_icon_theme_load_icon(icon_theme, stock_id,
					     TAB_LABEL_ICON_SIZE,
					     static_cast<GtkIconLookupFlags>(0),
					     NULL);
#if 0
  fprintf(stderr, "TabLabelIconSet::composite_stock_with_addons():\n");
  fprintf(stderr, "  stock_id=\"%s\"\n", stock_id);
  if (base != NULL)
    fprintf(stderr, "  icon from the theme: n_channels=%d, has_alpha=%d, width=%d, height=%d\n",
	    gdk_pixbuf_get_n_channels(base), gdk_pixbuf_get_has_alpha(base),
	    gdk_pixbuf_get_width(base), gdk_pixbuf_get_height(base));
  else
    fprintf(stderr, "  *** Cannot find the icon ***\n");
#endif

  ModPixbufSimpleAnim *anim = mod_pixbuf_simple_anim_new(16, 16, 2.0);
  mod_pixbuf_simple_anim_set_loop(anim, TRUE);
  mod_pixbuf_simple_anim_add_frame(anim, base);
  GdkPixbuf *frame = composite_pixbufs(base, manager.wait_addon);
  if (frame != NULL)
    {
      mod_pixbuf_simple_anim_add_frame(anim, frame);
      g_object_unref(frame);
    }
  wait_animation = GDK_PIXBUF_ANIMATION(anim);


  anim = mod_pixbuf_simple_anim_new(16, 16, 2.0);
  mod_pixbuf_simple_anim_set_loop(anim, TRUE);
  mod_pixbuf_simple_anim_add_frame(anim, base);
  frame = composite_pixbufs(base, manager.preparation_addon);
  if (frame != NULL)
    {
      mod_pixbuf_simple_anim_add_frame(anim, frame);
      g_object_unref(frame);
    }
  preparation_animation = GDK_PIXBUF_ANIMATION(anim);

  done_icon = composite_pixbufs(base, manager.done_addon);
  error_icon = composite_pixbufs(base, manager.error_addon);
  notice_icon = composite_pixbufs(base, manager.notice_addon);

  anim = mod_pixbuf_simple_anim_new(16, 16, 16.0);
  mod_pixbuf_simple_anim_set_loop(anim, TRUE);
  progress_animation = GDK_PIXBUF_ANIMATION(anim);

  for (int i = 0; i < OCHUSHA_NUMBER_OF_NET_PROGRESS_ADDONS; i++)
    {
      if (manager.progress_addons[i] != NULL)
	{
	  frame = composite_pixbufs(base, manager.progress_addons[i]);
	  if (frame != NULL)
	    {
	      mod_pixbuf_simple_anim_add_frame(anim, frame);
	      g_object_unref(frame);
	    }
	}
    }

  g_object_unref(base);
}


TabLabelManager::TabLabelManager(Controller &controller)
  : controller(controller),
    done_addon(NULL),
    error_addon(NULL),
    notice_addon(NULL),
    preparation_addon(NULL),
    wait_addon(NULL)
{
  for (int i = 0; i < OCHUSHA_NUMBER_OF_NET_PROGRESS_ADDONS; i++)
    progress_addons[i] = NULL;
}


TabLabelManager::~TabLabelManager()
{
  for (int i = 0; i < OCHUSHA_NUMBER_OF_NET_PROGRESS_ADDONS; i++)
    if (progress_addons[i] != NULL)
      {
	g_object_unref(progress_addons[i]);
	progress_addons[i] = NULL;
      }

  if (done_addon != NULL)
    {
      g_object_unref(done_addon);
      done_addon = NULL;
    }

  if (error_addon != NULL)
    {
      g_object_unref(error_addon);
      error_addon = NULL;
    }

  if (notice_addon != NULL)
    {
      g_object_unref(notice_addon);
      notice_addon = NULL;
    }

  if (preparation_addon != NULL)
    {
      g_object_unref(preparation_addon);
      preparation_addon = NULL;
    }

  list<TabLabelIconSet *>::iterator iter = icon_set_list.begin();
  list<TabLabelIconSet *>::iterator end_iter = icon_set_list.end();
  while (iter != end_iter)
    delete *iter++;
}


bool
TabLabelManager::prepare_addons()
{
  if (done_addon != NULL)
    return true;

  GtkWindow *toplevel = controller.get_toplevel();
  if (toplevel == NULL)
    return false;

  GtkWidget *widget = GTK_WIDGET(toplevel);
  g_return_val_if_fail(widget != NULL, false);

  GtkIconTheme *icon_theme;
  if (gtk_widget_has_screen(widget))
    icon_theme = gtk_icon_theme_get_for_screen(gtk_widget_get_screen(widget));
  else
    icon_theme = gtk_icon_theme_get_default();

  done_addon = gtk_icon_theme_load_icon(icon_theme,
					OCHUSHA_STOCK_NET_DONE_ADDON,
					TAB_LABEL_ICON_SIZE,
					static_cast<GtkIconLookupFlags>(0),
					NULL);
  g_return_val_if_fail(done_addon != NULL, false);

  error_addon = gtk_icon_theme_load_icon(icon_theme,
					 OCHUSHA_STOCK_NET_ERROR_ADDON,
					 TAB_LABEL_ICON_SIZE,
					 static_cast<GtkIconLookupFlags>(0),
					 NULL);
  g_return_val_if_fail(error_addon != NULL, false);

  notice_addon = gtk_icon_theme_load_icon(icon_theme,
					  OCHUSHA_STOCK_NET_NOTICE_ADDON,
					  TAB_LABEL_ICON_SIZE,
					  static_cast<GtkIconLookupFlags>(0),
					  NULL);
  g_return_val_if_fail(notice_addon != NULL, false);

  preparation_addon
    = gtk_icon_theme_load_icon(icon_theme,
			       OCHUSHA_STOCK_NET_PREPARATION_ADDON,
			       TAB_LABEL_ICON_SIZE,
			       static_cast<GtkIconLookupFlags>(0),
			       NULL);
  g_return_val_if_fail(preparation_addon != NULL, false);

  wait_addon = gtk_icon_theme_load_icon(icon_theme,
					OCHUSHA_STOCK_NET_WAIT_ADDON,
					TAB_LABEL_ICON_SIZE,
					static_cast<GtkIconLookupFlags>(0),
					NULL);
  g_return_val_if_fail(wait_addon != NULL, false);

  progress_addons[0]
    = gtk_icon_theme_load_icon(icon_theme,
			       OCHUSHA_STOCK_NET_PROGRESS_ADDON0,
			       TAB_LABEL_ICON_SIZE,
			       static_cast<GtkIconLookupFlags>(0),
			       NULL);
  g_return_val_if_fail(progress_addons[0] != NULL, false);

  progress_addons[1]
    = gtk_icon_theme_load_icon(icon_theme,
			       OCHUSHA_STOCK_NET_PROGRESS_ADDON1,
			       TAB_LABEL_ICON_SIZE,
			       static_cast<GtkIconLookupFlags>(0),
			       NULL);
  g_return_val_if_fail(progress_addons[1] != NULL, false);

  progress_addons[2]
    = gtk_icon_theme_load_icon(icon_theme,
			       OCHUSHA_STOCK_NET_PROGRESS_ADDON2,
			       TAB_LABEL_ICON_SIZE,
			       static_cast<GtkIconLookupFlags>(0),
			       NULL);
  g_return_val_if_fail(progress_addons[2] != NULL, false);

  progress_addons[3]
    = gtk_icon_theme_load_icon(icon_theme,
			       OCHUSHA_STOCK_NET_PROGRESS_ADDON3,
			       TAB_LABEL_ICON_SIZE,
			       static_cast<GtkIconLookupFlags>(0),
			       NULL);
  g_return_val_if_fail(progress_addons[3] != NULL, false);

  progress_addons[4]
    = gtk_icon_theme_load_icon(icon_theme,
			       OCHUSHA_STOCK_NET_PROGRESS_ADDON4,
			       TAB_LABEL_ICON_SIZE,
			       static_cast<GtkIconLookupFlags>(0),
			       NULL);
  g_return_val_if_fail(progress_addons[4] != NULL, false);

  progress_addons[5]
    = gtk_icon_theme_load_icon(icon_theme,
			       OCHUSHA_STOCK_NET_PROGRESS_ADDON5,
			       TAB_LABEL_ICON_SIZE,
			       static_cast<GtkIconLookupFlags>(0),
			       NULL);
  g_return_val_if_fail(progress_addons[5] != NULL, false);

  progress_addons[6]
    = gtk_icon_theme_load_icon(icon_theme,
			       OCHUSHA_STOCK_NET_PROGRESS_ADDON6,
			       TAB_LABEL_ICON_SIZE,
			       static_cast<GtkIconLookupFlags>(0),
			       NULL);
  g_return_val_if_fail(progress_addons[6] != NULL, false);

  progress_addons[7]
    = gtk_icon_theme_load_icon(icon_theme,
			       OCHUSHA_STOCK_NET_PROGRESS_ADDON7,
			       TAB_LABEL_ICON_SIZE,
			       static_cast<GtkIconLookupFlags>(0),
			       NULL);
  g_return_val_if_fail(progress_addons[7] != NULL, false);

  return done_addon != NULL;
}


TabLabel *
TabLabelManager::tab_label_new(const gchar *text, const gchar *stock_id)
{
  g_return_val_if_fail(stock_id != NULL, NULL);

  if (!prepare_addons())
    return NULL;

  list<TabLabelIconSet *>::iterator iter = icon_set_list.begin();
  list<TabLabelIconSet *>::iterator end_iter = icon_set_list.end();

  TabLabelIconSet *icon_set = NULL;
  while (iter != end_iter)
    {
      icon_set = *iter;

      if (strcmp(stock_id, icon_set->get_stock_id()) == 0)
	break;

      ++iter;
    }

  if (iter == end_iter)
    {
      icon_set = new TabLabelIconSet(stock_id, *this);
      icon_set_list.push_front(icon_set);
    }

  return new TabLabel(text, *icon_set);
}


TabLabel::TabLabel(const gchar *text, const TabLabelIconSet &icon_set)
  : icon_set(icon_set),
    in_error(false)
{
  label = OCHUSHA_ICON_LABEL(ochusha_icon_label_new(text));
  ochusha_icon_label_set_icon_from_animation(label,
					     icon_set.get_wait_animation());

  g_object_set_qdata(G_OBJECT(label), label_id, this);
  g_signal_connect(label, "destroy",
		   reinterpret_cast<GCallback>(icon_label_destroy_cb), this);
}


TabLabel::~TabLabel()
{
#if VERBOSE_DESTRUCTOR
  cerr << "TabLabel(@" << this << ") destructed.\n";
#endif
}


void
TabLabel::icon_label_destroy_cb(GtkWidget *widget, TabLabel *label)
{
  GDK_THREADS_LEAVE();

  delete label;

  GDK_THREADS_ENTER();
}


TabLabel *
TabLabel::get_label(GtkWidget *widget)
{
  return static_cast<TabLabel *>(g_object_get_qdata(G_OBJECT(widget),
						    label_id));
}


void
TabLabel::access_started()
{
  // sigslotʥϥɥ
  GDK_THREADS_ENTER();

  in_error = false;

  ochusha_icon_label_set_icon_from_animation(label,
					     icon_set.get_preparation_animation());

  GDK_THREADS_LEAVE();
}


void
TabLabel::access_progressed()
{
  // sigslotʥϥɥ
  GDK_THREADS_ENTER();

  ochusha_icon_label_set_icon_from_animation(label,
					     icon_set.get_progress_animation());

  GDK_THREADS_LEAVE();
}


void
TabLabel::access_finished()
{
  // sigslotʥϥɥ
  GDK_THREADS_ENTER();

  ochusha_icon_label_set_icon_from_pixbuf(label, icon_set.get_done_icon());

  GDK_THREADS_LEAVE();
}


void
TabLabel::access_failed()
{
  // sigslotʥϥɥ
  GDK_THREADS_ENTER();

  in_error = true;

  ochusha_icon_label_set_icon_from_pixbuf(label, icon_set.get_error_icon());

  GDK_THREADS_LEAVE();
}


void
TabLabel::access_terminated()
{
  // sigslotʥϥɥ
  GDK_THREADS_ENTER();

  in_error = true;

  ochusha_icon_label_set_icon_from_pixbuf(label, icon_set.get_error_icon());

  GDK_THREADS_LEAVE();
}


void
TabLabel::notice(bool value)
{
  if (value)
    ochusha_icon_label_set_icon_from_pixbuf(label, icon_set.get_notice_icon());
  else
    ochusha_icon_label_set_icon_from_pixbuf(label, icon_set.get_done_icon());
}

}
