/* -*- c++ -*-
 * Copyright (c) 2003-2008 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: tree_ui_impl.h,v 1.9 2008/10/13 10:29:58 fuyu Exp $
 */

#ifndef _TREE_UI_IMPL_H_
#define _TREE_UI_IMPL_H_

#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/ref_count.h"
#include "libochusha/sigslot.h"
#include "libochusha/tree.h"

#include "libochushagtk/tree_menu_helper.h"

#include <list>

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{

  enum TreeRowType {
    TREE_ROW_TYPE_UNKNOWN = 0,
    TREE_ROW_TYPE_ELEMENT,
    TREE_ROW_TYPE_EMPTY_FOLDER_INDICATOR,
    TREE_ROW_TYPE_INTERNAL_SEPARATOR,
    TREE_ROW_TYPE_OPEN_IN_TAB_MENU
  };


  enum TreeRowColumnIndex {
    TREE_ELEMENT_DATA = 0,
    TREE_ROW_TYPE,
    TREE_ROW_N_COLUMNS
  };


  class TreePathHandle
  {
  private:
    GtkTreePath *path;


  public:
    inline TreePathHandle(GtkTreePath *path = NULL) : path(path) {}
    inline ~TreePathHandle() { if (path != NULL) gtk_tree_path_free(path); }

    inline GtkTreePath *get_path() const { return path; }
    inline operator GtkTreePath *() const { return path; }
    inline GtkTreePath *operator->() const { return path; }
    inline TreePathHandle &operator=(GtkTreePath *value)
    { path = value; return *this; }
    inline TreePathHandle &operator=(const TreePathHandle &tph)
    { path = tph.path; return *this; }
  };


  class Controller;
  class DoOnSelectionHelper;
  class TreeItemPropertiesDialog;
  class TreeUI;
  class TreeView;

  class TreeUIImpl : public has_slots
  {
    friend class AddElementDialog;
    friend class AppendElementFunc;
    friend class DoOnSelectionHelper;
    friend class TreeItemPropertiesDialog;
    friend class TreeView;
    friend class TreeViewSelectionData;
    friend class FolderTreeView;

  protected:
    Controller &controller;


  private:
    smart_ptr<RootFolder> root_folder;

    GtkTreeStore *master_store;			// GtkTreeStore

    GtkTreeModel *view_model;			// ̾viewGtkTreeModel
    GtkTreeModel *menu_model;			// menu_helperGtkTreeModel
    GtkTreeModel *folder_model;			// եGtkTreeModel

    OchushaTreeMenuHelper *menu_helper;
    GtkMenuShell *menu_shell;

    GtkTreeSelection *last_selection;


  protected:
    TreeView *master_view;


    // ե򤹤륳ܥܥåɬ줿եΥꥹȡ
    // ǥեȤǤϥ롼ȥեäƤ롣
    list<smart_ptr<Folder> > important_folders;

    // ƤFolder *time_of_last_modifiedι߽˥Ȥꥹȡ
    // ǶῨäեnĤ򥳥ܥܥåȤ˻Ȥ
    list<smart_ptr<Folder> > all_folders;

    // TreeγǤФץѥƥsingletonȤ뤿˻Ȥ
    list<TreeItemPropertiesDialog *> dialogs;

    // master_storeФgtk_tree_model_clear()ȯ
    // GtkTreeModelΥʥɬפ˱̵뤹뤿Υե饰
    bool ignore_notify_folder_modified;


  public:
    static gboolean is_view_iter(GtkTreeModel *model, GtkTreeIter *iter,
				 gpointer data);
    static gboolean is_visible_menu_iter(GtkTreeModel *model, GtkTreeIter *iter,
					 gpointer data);
    static gboolean is_separator_iter(GtkTreeModel *model, GtkTreeIter *iter,
				      gpointer data);
    static gboolean is_sensitive_iter(GtkTreeModel *model, GtkTreeIter *iter,
				      gpointer data);
    static gboolean is_folder_iter(GtkTreeModel *model, GtkTreeIter *iter,
				   gpointer data);
    static const char *canonicalize_element_name(TreeElement *element);


  private:
    // modelmenu_model
    static void update_menu_entry(GtkMenuItem *menu_item, GtkLabel *menu_label,
				  GtkTreeModel *model, GtkTreeIter *iter,
				  gpointer data);
    static void menu_activate_cb(OchushaTreeMenuHelper *helper,
				 GtkTreeModel *model, GtkTreeIter *iter,
				 GtkTreePath *path, guint button,
				 TreeUIImpl *impl);
    // modelmenu_model


    // modelmaster_store
    static void row_deleted_cb(GtkTreeModel *model, GtkTreePath *path,
			       TreeUIImpl *impl);
    // modelmaster_store


    // sigslotʥϥɥ
    void tree_updated_cb(RootFolder *root);


    void popup_context_menu(GtkTreeSelection *selection, guint button,
			    guint32 event_time);
    void popup_folder_tree_context_menu(GtkTreeSelection *selection,
					guint button, guint32 event_time);


    Folder *new_folder(Folder *parent, TreeElement *sibling,
		       GtkTreeSelection *selection);


    TreeItemPropertiesDialog *lookup_dialog(const gchar *title,
					    TreeElement *element,
					    GtkTreeRowReference *row_reference);
    // NULLrow_referenceͿ줿硢TreeItemPropertiesDialog
    // Cancel줿Ǥrowǻؤrowmaster_store
    void open_properties(const gchar *title, TreeElement *element,
			 GtkTreeRowReference *row_reference);

    void notify_dialog_destroyed(TreeItemPropertiesDialog *dialog);
    void remove(GtkTreeRowReference *row_reference);
    void emit_row_changed_from_filter(GtkTreeModel *model, GtkTreeIter *iter);


    // folderall_foldersŬʰ֤ɲä롣
    void register_folder(Folder *folder);

    void notify_selection_changed(GtkTreeSelection *selection);


  protected:
    void register_important_folder(Folder *folder);

    // iterǼ줿GtkTreeModelŪǤбlibochushaTreeElement
    // ŪǤparentFolder *TreeElement *Ȥɤؤ롣
    bool locate_element(GtkTreeModel *model, GtkTreeIter *iter,
			Folder **parent_p, TreeElement **element_p);

    bool convert_iter_to_master_iter(GtkTreeModel *model,
				     GtkTreeIter *master_iter,
				     GtkTreeIter *iter);
    bool convert_master_iter_to_iter(GtkTreeModel *model,
				     GtkTreeIter *iter,
				     GtkTreeIter *master_iter);
    bool get_master_iter(GtkTreeModel *model, GtkTreeIter *master_iter,
			 GtkTreeRowReference *reference);

    bool move_folder(Folder *parent_folder, Folder *folder,
		     Folder *dest_folder, TreeElement *dest_element,
		     GtkTreeViewDropPosition position);
    bool move_element(Folder *src_folder, TreeElement *src_element,
		      Folder *dest_folder, TreeElement *dest_element,
		      GtkTreeViewDropPosition position);


    virtual const gchar *get_element_icon_stock_id(TreeElement *element,
						   gboolean expanded);
    virtual void reconstruct();

    virtual const char *get_popup_menu_root() = 0;
    virtual const char *get_folder_tree_popup_menu_root() = 0;
    virtual GtkWidget *get_menu_widget() = 0;
    virtual void open(TreeElement *element, bool in_tab, bool with_browser,
		      bool select_page) = 0;
    virtual void bookmark(TreeElement *element) = 0;
    virtual const char *get_url(TreeElement *element) = 0;
    virtual const char *get_ui_name() = 0;


    // ץѥƥȯԤ"response"ʥΥϥɥ餫
    // ƤФ롣
    // ǥեȤȯԤresponse_idϰʲ̤ꡣ
    // GTK_RESPONSE_OK:			OKܥ󤬲줿
    // GTK_RESPONSE_CANCEL:		Cancelܥ󤬲줿
    // GTK_RESPONSE_DELETE_EVENT:	Ĥܥ󤬲줿
    virtual void process_response(GtkDialog *dialog, int response_id,
				  TreeElement *element) = 0;
    // subclassdialog->vboxľ˿äƥץѥƥν򤹤롣
    // true֤ä鴰falseξϥǥեȤ(̾)ɲ
    // 롣
    virtual bool construct_properties_contents(GtkDialog *dialog,
					       TreeElement *element) = 0;
    virtual void open_properties(const gchar *title, TreeElement *element);
    virtual void open_properties(TreeElement *element);

    virtual bool show_all() const = 0;


    // src_folder/src_elementפ
    // drop_folder/drop_elementפΰpositionmoveͤΰ٤drop줿
    // Ȼפäн衣
    // ˤtrue֤
    virtual bool process_drop_data(Folder *src_folder, TreeElement *src_element,
				   Folder *drop_folder,
				   TreeElement *drop_element,
				   GtkTreeViewDropPosition position,
				   bool move) = 0;
    // uri
    // drop_folder/drop_elementפΰpositionmoveͤΰ٤drop줿
    // Ȼפäн衣
    // ˤtrue֤
    virtual bool process_drop_data(const char *uri, Folder *drop_folder,
				   TreeElement *drop_element,
				   GtkTreeViewDropPosition position) = 0;


  public:
    TreeUIImpl(Controller &controller, RootFolder *root);
    virtual ~TreeUIImpl();

    inline RootFolder *get_root_folder() { return root_folder; }
    inline Controller &get_controller() { return controller; }

    TreeView *create_tree_view(const gchar *title = NULL,
			       bool is_master_view = false);
    TreeView *create_folder_tree_view(const gchar *title = NULL);

    void add_element(TreeElement *element, const gchar *dialog_title);
    void new_element(Folder *folder, TreeElement *element);
    void remove_element(Folder *folder, TreeElement *element);

    void notify_folder_modified(Folder *folder);
    void notify_element_updated(TreeElement *element);

    void setup_menubar();


    virtual GtkWidget *get_widget();
    virtual void restore_session();
    virtual void prepare_session();


    // ʲTreeUIpopup menuGtkActionΥϥɥ餫ƤФ롣
    void open_selection(bool in_tab, bool with_browser);
    void new_folder_selection();
    void bookmark_selection();
    void toggle_hide_selection();
    void hide_selection();
    void kill_selection();
    void copy_url_selection();
    void copy_name_selection();
    void copy_name_url_selection();
    void open_properties_selection();
  };


}


#endif	/* __cplusplus */

#endif	/* _TREE_UI_IMPL_H_ */
