/* -*- c++ -*-
 * Copyright (c) 2003-2008 The Ochusha Project.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: view_manager.h,v 1.28 2008/10/16 13:29:27 fuyu Exp $
 */

#ifndef _VIEW_MANAGER_H_
#define _VIEW_MANAGER_H_


#ifdef __cplusplus

#if defined(__GNUG__) && !defined(__ICC)
#pragma interface
#endif

#include "libochusha/sigslot.h"

#include "post_ui.h"

#include "libochushagtk/hypertext_view.h"
#include "libochushagtk/searchable.h"
#include "libochushagtk/tree_selector_view.h"
#include "libochushagtk_lgpl/mod_notebook.h"

#include <gtk/gtk.h>


using namespace std;
namespace Ochusha
{
  class BBSThread;
  class BulletinBoard;
  class Controller;
  class MainWindow;
  class Preferences;

  class ViewManager : public has_slots
  {
    friend class MainWindow;

  private:
    static GQuark bulletin_board_id;
    static GQuark bbs_thread_id;

    Controller &controller;
    Preferences &prefs;

    GtkActionGroup *action_group;

    // ˥塼С˸줦륢ɬפ˱sensitive֤򹹿
    GtkAction *close_this_page;
    GtkAction *bookmark_this_page;
    GtkAction *find_in_this_page;
    GtkAction *refresh_this_page;
    GtkAction *page_info;

    GtkAction *close_board;
    GtkAction *close_thread;
    GtkAction *search_board;
    GtkAction *search_thread;
    GtkAction *refresh_board;
    GtkAction *refresh_thread;
    GtkAction *auto_refresh_thread;
    GtkAction *bookmark_board;
    GtkAction *bookmark_thread;
    GtkAction *board_info;
    GtkAction *thread_info;

    GtkAction *board_left;
    GtkAction *board_right;
    GtkAction *thread_left;
    GtkAction *thread_right;

    GtkListStore *board_entries;
    GtkAction *board_combo;

    GtkListStore *thread_entries;
    GtkAction *thread_combo;


    /* ()()ɽˤʤäƤ륳ƥ */
    GtkWidget *contents_pane;


    /* (ġСʤɤޤ)ɽʬΥƥ */
    GtkWidget *board_box;		// GtkVBox
    GtkWidget *board_toolbar;		// GtkToolbar


    ModNotebook *board_book;		// ()ɽѤModNotebook
    guint number_of_boards;


    // 夬졢ߥ쥹ǥʥƥ
    GtkWidget *thread_pane;		// GtkVPaned


    /* (ġСʤɤޤ)ɽʬΥƥ */
    GtkWidget *thread_box;		// GtkVBox
    GtkWidget *thread_toolbar;		// GtkToolbar


    ModNotebook *thread_book;		// ɽѤModNotebook
    guint number_of_threads;

    EmbededResponseEditor editor;


    GtkEntry *current_url_entry;

    GtkWidget *current_focus_widget;
    GtkWidget *last_event_widget;


    guint board_search_message_id;
    gint board_search_response_id;
    gint board_search_request_id;

    guint thread_search_message_id;
    gint thread_search_response_id;
    gint thread_search_request_id;


    GtkWidget *create_contents_pane();
    void set_current_focus_widget(GtkWidget *widget);

    void update_board_pane();
    void update_board_tab();
    void update_board_navigator();
    void update_thread_pane();
    void update_thread_tab();
    void update_thread_navigator();

    static void popup_menu_destroy_cb(gpointer data);
    gboolean show_popup_menu(GtkWidget *target, GdkEventButton *event);

    void close_nth_board_page(gint page);
    void close_nth_thread_page(gint page);

    static void notify_position_cb(GtkPaned *paned, GParamSpec *arg1,
				   ViewManager *view_manager);
    static void notify_position_set_cb(GtkPaned *paned, GParamSpec *arg1,
				       ViewManager *view_manager);

    // Ϣ
    static void board_search_response_cb(GtkWidget *widget, int response_id,
					 ViewManager *view_manager);
    static OchushaSearchable *board_search_request_cb(gpointer unused,
						      ViewManager *view_manager);
    void setup_board_search();

    static void thread_search_response_cb(GtkWidget *widget, int response_id,
					  ViewManager *view_manager);
    static OchushaSearchable *thread_search_request_cb(gpointer unused,
						       ViewManager *view_manager);
    void setup_thread_search();


    // ᥤ˥塼ѥʥϥɥ
    static void close_this_page_cb(GtkAction *action,
				   ViewManager *view_manager);
    static void bookmark_this_page_cb(GtkAction *action,
				      ViewManager *view_manager);
    static void find_in_this_page_cb(GtkAction *action,
				     ViewManager *view_manager);
    static void refresh_this_page_cb(GtkAction *action,
				     ViewManager *view_manager);
    static void page_info_cb(GtkAction *action, ViewManager *view_manager);

    static void current_url_entry_activate_cb(GtkEntry *entry,
					      ViewManager *view_manager);

    static void close_board_cb(GtkAction *action, ViewManager *view_manger);
    static void close_thread_cb(GtkAction *action, ViewManager *view_manger);
    static void search_board_cb(GtkAction *action, ViewManager *view_manager);
    static void search_thread_cb(GtkAction *action, ViewManager *view_manager);
    static void refresh_board_cb(GtkAction *action, ViewManager *view_manager);
    static void refresh_thread_cb(GtkAction *action, ViewManager *view_manager);
    static void auto_refresh_thread_cb(GtkAction *action,
				       ViewManager *view_manager);
    static void bookmark_board_cb(GtkAction *action, ViewManager *view_manager);
    static void bookmark_thread_cb(GtkAction *action,
				   ViewManager *view_manager);
    static void board_info_cb(GtkAction *action, ViewManager *view_manager);
    static void thread_info_cb(GtkAction *action, ViewManager *view_manager);


    // ֥֥åޡǤΥݥåץåץ˥塼ѥʥϥɥ

    static void bookmark_tab_cb(GtkAction *action, ViewManager *view_manager);
    static void refresh_tab_cb(GtkAction *action, ViewManager *view_manager);
    static void close_tab_cb(GtkAction *action, ViewManager *view_manager);
    static void auto_refresh_thread_tab_cb(GtkAction *action,
					   ViewManager *view_manager);
#if 1	// deplicated
    static void bookmark_board_tab_cb(GtkAction *action,
				      ViewManager *view_manager);
    static void refresh_board_tab_cb(GtkAction *action,
				     ViewManager *view_manager);
    static void close_board_tab_cb(GtkAction *action,
				   ViewManager *view_manager);
    static void bookmark_thread_tab_cb(GtkAction *action,
				       ViewManager *view_manager);
    static void refresh_thread_tab_cb(GtkAction *action,
				      ViewManager *view_manager);
    static void close_thread_tab_cb(GtkAction *action,
				    ViewManager *view_manager);
#endif	// ޤdeplicated


    // ѹѥʥϥɥ
    static void vertical_placement_toggled_cb(GtkToggleAction *action,
					      ViewManager *view_manager);
    static void board_tab_shrinkable_toggled_cb(GtkToggleAction *action,
						ViewManager *view_manager);
    static void board_tab_enable_tooltips_toggled_cb(GtkToggleAction *action,
						     ViewManager *view_manager);
    static void board_tab_always_show_toggled_cb(GtkToggleAction *action,
						 ViewManager *view_manager);
    static void board_pane_always_show_toggled_cb(GtkToggleAction *action,
						  ViewManager *view_manager);
    static void thread_tab_shrinkable_toggled_cb(GtkToggleAction *action,
						 ViewManager *view_manager);
    static void thread_tab_enable_tooltips_toggled_cb(GtkToggleAction *action,
						      ViewManager *view_manager);
    static void thread_tab_always_show_toggled_cb(GtkToggleAction *action,
						  ViewManager *view_manager);
    static void thread_pane_always_show_toggled_cb(GtkToggleAction *action,
						   ViewManager *view_manager);
    static void show_threadlist_toolbar_toggled_cb(GtkToggleAction *action,
						   ViewManager *view_manager);
    static void show_thread_toolbar_toggled_cb(GtkToggleAction *action,
					       ViewManager *view_manager);

    static void enable_embeded_response_editor_toggled_cb(
						GtkToggleAction *action,
						ViewManager *view_manager);

    static gboolean focus_in_event_cb(GtkWidget *widget, GdkEventFocus *event,
				      ViewManager *view_manager);
    static gboolean enter_notify_event_cb(GtkWidget *widget,
					  GdkEventCrossing *event,
					  ViewManager *view_manager);

    void notify_view_changed(ModNotebook *notebook, GtkWidget *new_page);
    void follow_focus();
    static void board_switch_page_cb(ModNotebook *notebook,
				     ModNotebookPage *page, guint page_num,
				     ViewManager *view_manager);
    static gboolean board_button_press_event_cb(ModNotebook *notebook,
						GdkEventButton *event,
						ViewManager *view_manager);
    static void board_start_interactive_search_cb(OchushaTreeSelectorView *view,
						  ViewManager *view_manager);
    static void thread_switch_page_cb(ModNotebook *notebook,
				      ModNotebookPage *page, guint page_num,
				      ViewManager *view_manager);
    static gboolean thread_button_press_event_cb(ModNotebook *notebook,
						 GdkEventButton *event,
						 ViewManager *view_manager);
    static void thread_start_interactive_search_cb(OchushaHypertextView *view,
						   OchushaSearchActionType action,
						   ViewManager *view_manager);

    // sigslotʥϥɥ
    void prefs_changed_cb(Preferences &prefs);


  public:
    ViewManager(Controller &controller);
    ~ViewManager();

    GtkWidget *get_widget();

    inline EmbededResponseEditor &get_embeded_response_editor()
    { return editor; }
    void close_current_focus_tab();

    GtkWidget *get_current_board_view();
    BulletinBoard *get_current_board();

    GtkWidget *get_current_thread_view();
    BBSThread *get_current_thread();

    void append_board_view(BulletinBoard *board,
			   GtkWidget *view, GtkWidget *tab_label,
			   bool in_tab, bool select_page);
    void append_thread_view(BBSThread *thread,
			    GtkWidget *view, GtkWidget *tab_label,
			    bool in_tab, bool select_page);

    GtkWidget *find_board_view(const BulletinBoard *board, bool select_page);
    GtkWidget *get_board_tab_label(GtkWidget *view);

    GtkWidget *find_thread_view(const BBSThread *thread, bool select_page);
    GtkWidget *get_thread_tab_label(GtkWidget *view);

    GtkWidget *get_tab_label(const BBSThread *thread);
    GtkWidget *get_tab_label(const BulletinBoard *board);

    typedef void (*ForeachBoardFunc)(GtkWidget *view, BulletinBoard *board,
				     void *callback_data);
    void foreach_board(ForeachBoardFunc callback, void *callback_data);

    typedef void (*ForeachThreadFunc)(GtkWidget *view, BBSThread *thread,
				      void *callback_data);
    void foreach_thread(ForeachThreadFunc callback, void *callback_data);

    static BulletinBoard *get_board(GtkWidget *view);
    static BBSThread *get_thread(GtkWidget *view);
  };


}


#endif	/* __cplusplus */

#endif	/* _VIEW_MANAGER_H_ */
